# ============================================================================================================
# History.py
# lheywang on 21/12/2024
#
# Base file for the history class
#
# ============================================================================================================
import datetime

from ..BaseOptionnalClass import SiglentBase


class SiglentHistory(SiglentBase):
    """
    pySDS [History][SiglentHistory] :   Class herited from SiglentBase.
                                        Store all command related the control of history display.

        Attributes :
            Herited from SiglentBase

        Methods :
            Private (0) :
                None

            Public (7):
                EnableHistory :                 Enable history mode
                DisableHistory :                Disable history mode
                SetCurrentFrame :               Configure current frame
                GetFrameAcquisitionTime :       GetAcquisitionTime for a defined frame
                EnableHistoryList :             Enable List view
                DisableHistoryList :            Disable List view
    """

    def EnableHistory(self):
        """
        pySDS [History][EnableHistory] : Enable the History mode

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("HSMD ON")
        return self.__baseclass__.GetAllErrors()

    def DisableHistory(self):
        """
        pySDS [History][DisableHistory] : Disable the History mode

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("HSMD OFF")
        return self.__baseclass__.GetAllErrors()

    def SetCurrentFrame(self, Number):
        """
        pySDS [History][SetCurrentFrame] : Configure the current frame where the data is stored

            Arguments :
                Number : Integer of the position. Generally between 0 and 7960, but upper value can be influed by hardware options, timebase or resolution.

            Returns :
                self.GetAllErrors() : List of errors
        """
        self.__instr__.write(f"FRAM {Number}")
        return self.__baseclass__.GetAllErrors()

    def GetFrameAcquisitionTime(self):
        """
        pySDS [History][GetFrameAcquisitionTime] : Return the acquision time for this frame

            Arguments :
                None

            Returns :
                Duration under the form of a Python Time object (from Datetime package)
        """

        ret = self.__instr__.query("FTIM?").strip()

        if ret.split(" ")[0] == "FTIM":
            ret = ret.split(" ")[-1].split(":")
            return datetime.time(
                hour=ret[0].strip(),
                minute=ret[1].strip(),
                second=ret[2].strip(),
                microsecond=ret[3].strip(),
            )

        else:
            # Get measurement parameters
            samples = int(ret.replace(r"\x", ""), 16)
            sampling = 1 / int(self.__baseclass__.Acquistion.GetSampleRate()[:-3])

            # Measure duration
            duration = samples * sampling

            return datetime.time(microsecond=duration / 1000)

    def EnableHistoryList(self):
        """
        pySDS [History][EnableHistoryList] : Enable the list mode

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("HSLST ON")
        return self.__baseclass__.GetAllErrors()

    def DisableHistoryList(self):
        """
        pySDS [History][DisableHistoryList] : Disable the list mode

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.query("HSLST OFF")
        return self.__baseclass__.GetAllErrors()



# ============================================================================================================
# Decode.py
# lheywang on 21/12/2024
#
# Base file for the decode class
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase
from ..channel import SiglentChannel
from ..digital import SiglentDChannel


class SiglentDecode(SiglentBase):
    """
    pySDS [Decode][SiglentDecode] : Class herited from SiglentBase.
                                    Store all command related the control of digital bus decoding
        Attributes :
            Herited from SiglentBase

        Methods :
            Private (0) :
                None

            Public (8):
                EnableDecode :          Enable the bus decoding ability
                DisableDecode :         Disable the bus decoding ability
                ConfigureDecode :       Configure the global bus decoding behavior
                ConfigureI2CDecode :    Configure the I2C Decoding
                ConfigureSPIDecode :    Configure the SPI Decoding
                ConfigureUARTDecode :   Configure the UART Decoding
                ConfigureCANDecode :    Configure the CAN Decoding
                ConfigureLINDecode :    Configure the LIN Decoding
    """

    def EnableDecode(self):
        """
        pySDS [Decode][EnableDecode] : Enable the decode option on the scope

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("DCST ON")
        return self.__baseclass__.GetAllErrors()

    def DisableDecode(self):
        """
        pySDS [Decode][DisableDecode] : Disable the decode option on the scope

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("DCST OFF")
        return self.__baseclass__.GetAllErrors()

    def ConfigureDecode(
        self, Bus: int, Format="HEX", List="OFF", Link="", Scroll=0, Lines=0
    ):
        """
        pySDS [Decode][ConfigureDecode] : Configure the decode engine

            Arguments :
                Bus : 1 | 2 (Bus number)
                Format : BIN | DEC | HEX (Mode of print on screen) Default to HEX
                List : OFF | D1 | D2 Default to OFF
                Link : TR_TO_DC | DC_TO_TR
                Scroll : Integer
                Lines : Integer

            Returns :
                GetAllErrors() : List of errors
                or
                [n, Error codes] if errors occured within the function

            Errors codes :
                -1 :    Invalid Lines number
                -2 :    Invalid scroll number (must be > 0 and < Lines)
                -3 :    Invalid Link mode
                -4 :    Invalid List mode
                -5 :    Invalid format
                -6 :    Invalid Bus
        """
        # cmdeters check
        if Lines < 0 or Lines > 7:
            return [1, -1]
        if Scroll < 0 or Scroll > Lines:
            return [1, -2]
        if Link not in ["TR_TO_DC", "DC_TO_TR", ""]:
            return [1, -3]
        if List not in ["OFF", "D1", "D2"]:
            return [1, -4]
        if Format not in ["HEX", "DEC", "BIN"]:
            return [1, -5]
        if Bus < 1 or Bus > 2:
            return [1, -6]

        # Create the command string
        cmd = f"BUS,B{Bus},LIST,{List},FOMT,{Format}"
        if Link != "":
            cmd += f",LINK,{Link}"
        if Scroll != 0:
            cmd += f",LSSC,{List}"
        if Lines != 0:
            cmd += f",LSNM,{Lines}"

        self.__instr__.write(f"DCPA {cmd}")
        return self.__baseclass__.GetAllErrors()

    def ConfigureI2CDecode(
        self,
        Bus: int,
        SCL,
        SDA,
        SCLT: float = 1.65,
        SDAT: float = 1.65,
        Display="ON",
        RW="ON",
    ):
        """
        pySDS [Decode][ConfigureI2CDecode] : Configure the I2C decoding

        WARNING : If digital channel is provided, trigger level will be ignored

            Arguments :
                Bus :       Bus number, as in integer
                SCL :       SCL input. May be SiglentChannel or SiglentDigital class
                SDA :       SCL input. May be SiglentChannel or SiglentDigital class
                SCLT :      Trigger Level for SCL                                           Default to 1.65
                SDAT :      Trigger Level for SCL                                           Default to 1.65
                Display :   ON | OFF Display the bus decoding result                        Default to ON
                RW :        ON | OFF Display the Read/Write bit inside of adress, or not    Default to ON

            Returns :
                GetAllErrors() : List of errors
                or
                List or errors codes within the function

            Errors codes :
                -1 :    Wrong object passed on SCL
                -2 :    Wrong object passed on SDA
                -3 :    Invalid mode for Display
                -4 :    Invalid mode for RW
                -5 :    Invalid bus number
        """

        # parameters check
        if (type(SCL) is not SiglentChannel) and (type(SCL) is not SiglentDChannel):
            return [1, -1]
        if (type(SDA) is not SiglentChannel) and (type(SDA) is not SiglentDChannel):
            return [1, -2]
        if Display not in ["ON", "OFF"]:
            return [1, -3]
        if RW not in ["ON", "OFF"]:
            return [1, -4]
        if Bus < 1 or Bus > 2:
            return [1, -5]

        # Command execution
        cmd = f"B{Bus}:DCIC "

        cmd += f"SCL,{SCL.__channel__}"
        if type(SCL) == SiglentChannel:
            cmd += f",SCLT,{SCLT}"

        cmd += f",SDA,{SDA.__channel__}"
        if type(SDA) == SiglentChannel:
            cmd += f",SDAT,{SDAT}"

        cmd += f",DIS,{Display},RW,{RW}"

        self.__instr__.write(cmd)

        # Returns
        return self.__baseclass__.GetAllErrors()

    def ConfigureSPIDecode(
        self,
        Bus: int,
        CLK,
        MOSI,
        MISO,
        CS,
        CSMode="CS",
        Edge="RISING",
        Bit="MSB",
        Len=8,
        CLKT=1.65,
        MOSIT=1.65,
        MISOT=1.65,
        CST=1.65,
        CSTimeout=2,
        Display="ON",
    ):
        """
        pySDS [Decode][ConfigureSPIDecode] : Configure the decoding of the SPI Bus

        WARNING : If a digital channel is provided, it's trigger value will be ignored

            Arguments  :
                Bus :       1 | 2 Bus used to decode
                CLK :       Channel for CLK. May be a channel or a digital channel
                MOSI :      Channel for MOSI. May be a channel or a digital channel
                MISO :      Channel for MISO. May be a channel or a digital channel
                CS :        Channel for CS. May be a channel or a digital channel
                CSMode :    CS | NCS | TIMEOUT Configure the CS operation mode                                          Defaults to CS
                Edge :      RISING | FALLING Configure the mode of SPI between mode 0, 2 : Falling or 1, 3 : Rising     Defaults to RISING
                Bit :       MSB | LSB Configure the bit order                                                           Defaults to MSB
                Len :       Configure the transfer length                                                               Defaults to 8 bit
                CLKT :      Configure the trigger level of the CLK.                                                     Defaults to 1.65.
                MOSIT :     Configure the trigger level of the MOSI.                                                    Defaults to 1.65.
                MISOT :     Configure the trigger level of the MISO.                                                    Defaults to 1.65.
                CST :       Configure the trigger level of the CS.                                                      Defaults to 1.65.
                CSTimeout : Timeout value for the CS when in timeout mode                                               Defaults to 2 (us)
                Display :   ON | OFF Display the result on the screen                                                   Defaults to ON

            Returns :
                GetAllErrors() : List of errors
                Or
                List of errors that occured within the function

            Errors codes :
                -1 :    Incorrect type passed for CLK. Waiting for a SiglentChannel or SiglentDChannel
                -2 :    Incorrect type passed for MOSI. Waiting for a SiglentChannel or SiglentDChannel
                -3 :    Incorrect type passed for MISO. Waiting for a SiglentChannel or SiglentDChannel
                -4 :    Incorrect type passed for CS. Waiting for a SiglentChannel or SiglentDChannel
                -5 :    Invalid CS Mode
                -6 :    Invalid Bus
                -7 :    Invalid Edge provided
                -8 :    Invalid bit provided
                -9 :    Len is out of bounds (4-32)
                -10 :   Invalid display mode
        """
        # parameters check
        if (type(CLK) is not SiglentChannel) and (type(CLK) is not SiglentDChannel):
            return [1, -1]
        if (type(MOSI) is not SiglentChannel) and (type(MOSI) is not SiglentDChannel):
            return [1, -2]
        if (type(MISO) is not SiglentChannel) and (type(MISO) is not SiglentDChannel):
            return [1, -3]
        if (type(CS) is not SiglentChannel) and (type(CS) is not SiglentDChannel):
            return [1, -4]
        if CSMode not in ["CS", "NCS", "TIMEOUT"]:
            return [1, -5]
        if Bus < 1 or Bus > 2:
            return [1, -6]
        if Edge not in ["RISING", "FALLING"]:
            return [1, -7]
        if Bit not in ["MSB", "LSB"]:
            return [1, -8]
        if Len < 4 or Len > 32:
            return [1, -9]
        if Display not in ["ON", "OFF"]:
            return [1, -10]

        # Creating command
        cmd = f"B{Bus}:DCSP ,MISO,{MISO.__channel__},MOSI,{MOSI.__channel__}"

        cmd += f"CLK,{CLK.__channel__}"
        if type(CLK) == SiglentChannel:
            cmd += f",CLKT,{CLKT}"
        cmd += f",MISO,{MISO.__channel__}"
        if type(MISO) == SiglentChannel:
            cmd += f",MISOT,{MISOT}"
        cmd += f",MOSI,{MOSI.__channel__}"
        if type(MOSI) == SiglentChannel:
            cmd += f",MOSIT,{MOSIT}"

        # Acting depending the chip select mode
        if CSMode == "CS":
            cmd += f",CS,{CS.__channel__}"
            if type(CST) == SiglentChannel:
                cmd += f",CST,{CST}"

        elif CSMode == "NCS":
            cmd += f",NCS,{CS.__channel__}"
            if type(CST) == SiglentChannel:
                cmd += f",NCST,{CST}"

        elif CSMode == "TIMEOUT":
            cmd += f",TIM,{CSTimeout}"

        cmd += f",EDGE,{Edge},DLEN,{Len},BIT,{Bit},DIS,{Display}"

        # Executing the command
        self.__instr__.write(cmd)

        # Returns errors
        return self.__baseclass__.GetAllErrors()

    def ConfigureUARTDecode(
        self,
        Bus: int,
        RX,
        TX,
        Baud=115200,
        DLEN=8,
        Parity="NONE",
        Stop=1,
        Polarity="HIGH",
        Bit="MSB",
        RXT=1.65,
        TXT=1.65,
        Display="ON",
    ):
        """
        pySDS [Decode][ConfigureUARTDecode] : Configure the UART Decoding

        WARNING : If a SiglentDChannel is provided, then the associated trigger value will be ignored.

            Arguments :
                Bus :           Select the number of the bus decoder used. 1 | 2
                RX :            SiglentChannel or SiglentDChannel associated with RX
                TX :            SiglentChannel or SiglentDChannel associated with TX
                Baud :          Baud rate.                                              Default to 115200
                DLEN :          Data len.  5 <= Val <= 8                                Default to 8
                Parity :        Parity bit. NONE | EVEN | ODD                           Default to NONE
                Stop :          Number of stop bits. 1 | 1.5 | 2                        Default to 1
                Polarity :      Polarity of the signal. LOW | HIGH                      Default to HIGH
                Bit :           Order of bits on the message. MSB | LSB                 Default to MSB
                RXT :           RX Trigger level.                                       Default to 1.65
                TXT :           TX Trigger level.                                       Default to 1.65
                Dsiplay :       Display the bus or not ON | OFF                         Default to ON

            Returns :
                GetAllErrors() : List of errors
                or
                List of errors that occured within the function execution, presented in the same way.

            Errors codes
                -1 :    Invalid type passed for RX
                -2 :    Invalid type passed for TX
                -3 :    Invalid Parity mode
                -4 :    Invalid stop bit number
                -5 :    Invalid polarity
                -6 :    Invalid bit order
                -7 :    Invalid display mode
                -8 :    Invalid bus number
                -9 :    Invalid data len
                -10 :   Invalid baud rate
        """
        # parameters check
        if (type(RX) is not SiglentChannel) and (type(RX) is not SiglentDChannel):
            return [1, -1]
        if (type(TX) is not SiglentChannel) and (type(TX) is not SiglentDChannel):
            return [1, -2]

        if Parity not in ["NONE", "ODD", "EVEN"]:
            return [1, -3]
        if Stop not in [1, 1.5, 2]:
            return [1, -4]
        if Polarity not in ["HIGH", "LOW"]:
            return [1, -5]
        if Bit not in ["MSB", "LSB"]:
            return [1, -6]
        if Display not in ["ON", "OFF"]:
            return [1, -7]

        if Bus < 1 or Bus > 2:
            return [1, -8]
        if DLEN < 5 or DLEN > 8:
            return [1, -9]
        if Baud < 300 or Baud > 50_000_000:
            return [1, -10]

        # Command creation
        cmd = f"B{Bus}:DCUT "

        cmd += f",RX,{RX.__channel__}"
        if type(RX) == SiglentChannel:
            cmd += f",RXT,{RXT}"

        cmd += f",TX,{TX.__channel__}"
        if type(TX) == SiglentChannel:
            cmd += f",TXT,{TXT}"

        cmd += f",BAUD,{Baud},DLEN,{DLEN},PAR,{Parity},STOP,{Stop},POL,{Polarity},BIT,{Bit},DIS,{Display}"

        # command execution
        self.__instr__.write(cmd)

        # Returns
        return self.__baseclass__.GetAllErrors()

    def ConfigureCANDecode(
        self, Bus: int, CANH, CANL, SRC, Baud, CANHT=1.65, CANLT=1.65, Display="ON"
    ):
        """
        pySDS [Decode][ConfigureCANDecode] : Configure the CAN decoding operation

        WARNING : If a SiglentDChannel is provided, then the associated trigger value will be ignored.

            Arguments :
                Bus :       ID of the bus where to place the decode results
                CANH :      CANH associated SiglentChannel or SiglentDChannel.
                CANL :      CANL associated SiglentChannel or SiglentDChannel.
                SRC :       Source for decoding. CANH_H | CAN_L | SUB_L.
                Baud :      Baud rate of the communication.
                CANHT :     Threshold for the CANH pin.                             Default to 1.65
                CANLT :     Threshold for the CANL pin.                             Default to 1.65
                Display :   Shall we display the decoding on the screen ?           Default to ON

            Returns :
                GetAllErrors() : List of errors
                or
                List of errors that occured within the function execution, formatted in the same way

            Errors codes :
                -1 :    Invalid CANH type passed
                -2 :    Invalid CANL type passed
                -3 :    Invalid SRC
                -4 :    Invalid DISPLAY setting.
                -5 :    Invalid bus ID
                -6 :    Invalid BAUD RATE
        """
        # parameters check
        if (type(CANH) is not SiglentChannel) and (type(CANH) is not SiglentDChannel):
            return [1, -1]
        if (type(CANL) is not SiglentChannel) and (type(CANL) is not SiglentDChannel):
            return [1, -2]

        if SRC not in ["CAN_H", "CAN_L", "SUB_L"]:
            return [1, -3]
        if Display not in ["ON", "OFF"]:
            return [1, -4]

        if Bus < 1 or Bus > 2:
            return [1, -5]
        if Baud < 5_000 or Baud > 1_000_000:
            return [1, -6]

        # Command creation
        cmd = f"B{Bus}:DCCN "

        cmd += f",CANH,{CANH.__channel__}"
        if type(CANH) == SiglentChannel:
            cmd += f",CANHT,{CANHT}"

        cmd += f",CANL,{CANL.__channel__}"
        if type(CANL) == SiglentChannel:
            cmd += f",CANLT,{CANLT}"

        cmd += f",BAUD,{Baud},SRC,{SRC},DIS,{Display}"

        # command execution
        self.__instr__.write(cmd)

        # Returns
        return self.__baseclass__.GetAllErrors()

    def ConfigureLINDecode(self, Bus: int, SRC, Baud, SRCT=1.65, Display="ON"):
        """
        pySDS [Decode][ConfigureLINDecode] : Configure the LIn decoding operation

        WARNING : If a SiglentDChannel is provided, then the associated trigger value will be ignored.

            Arguments :
                Bus :       ID of the bus where to place the decode results
                SRC :       SiglentChannel or SiglentDChannel associated with the SRC Pin
                Baud :      Baud rate of the communication
                SRCT :      Threshold the SRC Pin.                                              Default to 1.65
                Display :   Shall we show the result on the screen ?                            Default to ON

            Returns :
                GetAllErrors() : List of errors
                or
                List of errors that occured within the function execution, formatted in the same way

            Errors codes :
                -1 :    Invalid SRC type passed
                -2 :    Invalid DISPLAY setting.
                -3 :    Invalid bus ID
                -4 :    Invalid BAUD RATE
        """
        # parameters check
        if (type(SRC) is not SiglentChannel) and (type(SRC) is not SiglentDChannel):
            return [1, -1]

        if Display not in ["ON", "OFF"]:
            return [1, -2]

        if Bus < 1 or Bus > 2:
            return [1, -3]
        if Baud < 300 or Baud > 2_000:
            return [1, -4]

        # Command creation
        cmd = f"B{Bus}:DCLN "

        cmd += f",SRC,{SRC.__channel__}"
        if type(SRC) == SiglentChannel:
            cmd += f",SRCT,{SRCT}"

        cmd += f",BAUD,{Baud},DIS,{Display}"

        # command execution
        self.__instr__.write(cmd)

        # Returns
        return self.__baseclass__.GetAllErrors()



# ============================================================================================================
# Digital.py
# lheywang on 21/12/2024
#
# Base file for the digital class
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase
from .digitalchannel import SiglentDChannel


class SiglentDigital(SiglentBase):
    """
    pySDS [Digital][SiglentDigital] :   Class herited from SiglentBase.
                                        Store all command related the control of digital channels (scopes with MSO option only)

        WARNING :   This section of code is extremely undocumented, and the rare parts that are documentation aren't comprehesible.
                    Take all of this code with salt, since bugs can exists here due to an error of my person, or an error of the doc, or both
                    If something went bugging here, you can report it !

        Attributes :
            Herited from SiglentBase

        Methods :
            Private (0) :
                None

            Public (5):
                EnableDigitalChannel :          Enable a channel
                DisableDigitalChannel :         Disable a channel
                SetDigitalChannelThreshold :    Configure threshold for a group of channels
                EnableDigital :                 Enable global digital engine
                DisableDigital :                Disable global digital engine
    """

    def EnableDigitalChannel(self, Channel: SiglentDChannel):
        """
        pySDS [Digital][EnableDigitalChannel] : Enable a digital channel and display it on the screen

            Arguments :
                Channel : SiglentDChannel to be enabled and shown

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write(f"{Channel.__channel__}:DGCH ON")
        return self.__baseclass__.GetAllErrors()

    def DisableDigitalChannel(self, Channel: SiglentDChannel):
        """
        pySDS [Digital][DisableDigitalChannel] : Disable a digital channel and display it on the screen

            Arguments :
                Channel : SiglentDChannel to be disabled and hide

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write(f"{Channel.__channel__}:DGCH OFF")
        return self.__baseclass__.GetAllErrors()

    def SetDigitalChannelThreshold(self, Group: int, Threshold, Value=0):
        """
        pySDS [Digital][SetDigitalChannelThreshold] : Configure the threshold for a type of logic signals.

        WARNING : This setting is applied for a group of 8 channels. There is only two groups.

            Arguments :
                Group : 1 | 2 Group where the setting is applied
                Threshold : TTL | CMOS | CMOS3.3 | CMOS2.5 | CUSTOM
                Value : Only for CUSTOM, the value in V between -5 and 5

            Returns :
                self.GetAllErrors() : List of errors
                or
                Errors codes

            Errors codes :
                -1 : Invalid group
                -2 : Invalid Threshold type
                -3 : Invalid value
        """
        if Group not in [1, 2]:
            return [1, -1]
        if Threshold not in ["TTL", "CMOS", "CMOS3.3", "CMOS2.5", "CUSTOM"]:
            return [1, -2]
        if Value < -5 or Value > 5:
            return [1, -3]

        cmd = f"C{Group}:DGTH {Threshold}"
        if Threshold == "CUSTOM":
            cmd += f",{Value}V"

        self.__instr__.write(cmd)
        return self.__baseclass__.GetAllErrors()

    def EnableDigital(self):
        """
        pySDS [Digital][EnableDigital] : Enable the digital section on the scope

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("DI:SW ON")
        return self.__baseclass__.GetAllErrors()

    def DisableDigital(self):
        """
        pySDS [Digital][DisableDigital] : Disable the digital section on the scope

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.query("DI:SW OFF")
        return self.__baseclass__.GetAllErrors()



# Aux files !
# Others files
from .acquisition import SiglentAcquisition
from .channel import SiglentChannel
from .communication import SiglentCommunication
from .cursor import SiglentCursor
from .decode import SiglentDecode
from .deprecated_functions import ACAL, DATE, COUNTER
from .digital import SiglentDigital
from .display import SiglentScreen
from .Files import SiglentFiles
from .generics import SCPIGenerics
from .history import SiglentHistory
from .maths import SiglentMaths
from .measure import SiglentMeasure
from .passfail import SiglentPassFail
from .references import SiglentReference
from .timebase import SiglentTimebase
from .trigger import SiglentTrigger
from .waveform import SiglentWaveform

# Main init.py file !
from .pySDS import PySDS


# ============================================================================================================
# LIN.py
# lheywang on 02/01/2025
#
# Advanced file for the trigger class, specialized on LIN bus
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase
from ..channel import SiglentChannel
from ..digital import SiglentDChannel


class SiglentLIN(SiglentBase):
    """
    pySDS [Trigger][SiglentLIN] :   Class herited from SiglentBase.
                                    Store all command related the control of the triggering system for the LIN bus

        Attributes :
            Herited from SiglentBase

        Methods :
            Private (0) :
                None

            Public (15):

    """

    def SetTriggerOnSRC(self, Channel, Threshold=1.65):
        """
        pySDS [LIN][SetTriggerOnSRC] : Configure the trigger on the LIN pin

            Arguments :
                Channel :       SiglentChannel or SiglentDCHannel related to the LIN pin
                Threshold :     For analog channel only, the used voltage. Default to 1.65

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Channel
        """
        if type(Channel) is not SiglentChannel and type(Channel) is not SiglentDChannel:
            return [1, -1]

        cmd = f"TRLIN:SRC {Channel.__channel__}"

        if type(Channel) == SiglentChannel:
            cmd += f",{Threshold}"

        self.__instr__.write(cmd)
        return self.__baseclass__.GetAllErrors()

    def SetTriggerOnCondition(self, Condition: str):
        """
        pySDS [LIN][SetTriggerOnCondition] : Configure the trigger on a condition on the CAN Bus

            Arguments :
                Condition : BREAK | DATA_ERROR | ID | ID_AND_DATA

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid condition

            Possibles conditions :
                BREAK— Break condition.
                ID— Specify a search based on ID.
                ID_AND_DATA—Specify a search based on ID and data.
                DATA_ERROR— Error frame.
        """
        if Condition not in ["BREAK", "DATA_ERROR", "ID", "ID_AND_DATA"]:
            return [1, -1]

        self.__instr__.write(f"TRLIN:CON {Condition}")
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerID(self, ID: int):
        """
        pySDS [LIN][ConfigureTriggerID] : Set the ID for ID and ID_AND_DATA mode of trigger

            Arguments :
                ID : Integer between 0-64

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid ID
        """
        if ID < 0 or ID > 64:
            return [1, -1]

        self.__instr__.write(f"TRLIN:ID {ID}")
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerData1(self, Value: int):
        """
        pySDS [LIN][ConfigureTriggerData1] : Configure the data (first byte) to be used to trigger for ID_AND_DATA mode.

            Arguments :
                Value: 0-255 value

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid value
        """
        if Value < 0 or Value > 255:
            return [1, -1]

        self.__instr__.write(f"TRLIN:DATA {Value}")
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerData2(self, Value: int):
        """
        pySDS [LIN][ConfigureTriggerData2] : Configure the data (second byte) to be used to trigger for ID_AND_DATA mode.

            Arguments :
                Value: 0-255 value

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid value
        """
        if Value < 0 or Value > 255:
            return [1, -1]

        self.__instr__.write(f"TRLIN:DAT2 {Value}")
        return self.__baseclass__.GetAllErrors()

    def SetTriggerBaud(self, Baud):
        """
        pySDS [LIN][SetTriggerBaud] : Set the baud rate of the UART comm

            Arguments :
                Baud : Value. (As a string for standard values, or as an integer for non standard)*

                * Sending a standard as integer will be seen as custom but will also works fine !

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Baud

            Standard baud values :
                600
                1200
                2400
                4800
                9600
                19200
        """
        if Baud < 300 or Baud > 20_000:
            return [1, -1]

        if Baud not in [600, 1200, 2400, 4800, 9600, 19200]:
            cmd = f"TRLIN:BAUD CUSTOM,{Baud}"
        else:
            cmd = f"TRLIN:BAUD {Baud}"

        self.__instr__.write(cmd)
        return self.__baseclass__.GetAllErrors()



# ============================================================================================================
# Channel.py
# lheywang on 17/12/2024
#
# Base file for the channel class
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase


class SiglentChannel(SiglentBase):
    """
    pySDS [Channel][SiglentChannel] :   Class herited from SiglentBase.
                                        Store all command related to a channel
        Attributes :
            Herited from SiglentBase

        Methods :
            Private (0) :
                None

            Public (12):
                SetAttenuation :            Configure channel attenuation
                EnableBandwithFilter :      Enable 20 MHz filter on the channel
                DisableBandwithFilter :     Disable 20 MHz filter on the channel
                SetCoupling :               Configure channel coupling
                SetOffset :                 Configure channel offset
                SetSkew :                   Configure channel skew
                EnableTrace :               Enable trace draw on the screen
                DisableTrace :              Disable trace draw on the screen
                SetTraceUnit :              Configure trace unit
                SetTraceDIV :               Configure trace gain
                EnableTraceInvert :         Enable inversion of the trace
                DisableTraceInvert :        Disable inversion of the trace
    """

    def __init__(self, instr, baseclass, channel, impedance):
        """
        Overhide the standard class init to store some more advanced data !

        Check SiglentBase doc before !

        Added attributes :
            Private (2) :
                __channel__ :   Descriptor of the channel
                __impedance__ : List of supported impedance for this channel

            Public (0) :
                None

        Added methods :
            Private (0) :
                None

            Public (0) :
                None
        """
        super().__init__(instr, baseclass)
        self.__channel__ = channel
        self.__impedance__ = impedance

    def SetAttenuation(self, Value):
        """
        pySDS [Channel][SetAttenuation] : Configure the probe attenuation

            Arguments :
                Value : The attenuation value, between 0.1 and 10000 (checked against a list of values)

            Returns :
                self.GetAllErrors()
        """
        if Value not in [
            0.1,
            0.2,
            0.5,
            1,
            2,
            5,
            10,
            20,
            50,
            100,
            200,
            500,
            1000,
            2000,
            5000,
            10000,
        ]:
            return -1

        self.__instr__.write(f"{self.__channel__}:ATTN {Value}")
        return self.__baseclass__.GetAllErrors()

    def EnableBandwithFilter(self):
        """
        pySDS [Channel][EnableBandwithFilter] : Enable a 20 Mhz low pass filter on the channel. Used to rejet high frequency noise.

            Arguments :
                None__

            Returns :
                self.GetAllErrors()
        """

        self.__instr__.write(f"BWL {self.__channel__},ON")
        return self.__baseclass__.GetAllErrors()

    def DisableBandwithFilter(self):
        """
        pySDS [Channel][DisableBandwithFilter] : Disable a 20 Mhz low pass filter on the channel.

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write(f"BWL {self.__channel__},OFF")
        return self.__baseclass__.GetAllErrors()

    def SetCoupling(self, ACDC="D", Impedance=1000000):
        """
        pySDS [Channel][SetCoupling] : Set the channel coupling mode.

            Arguments :
                ACDC : Mode of coupling, AC or DC (AC rejet any DC signal). Values are A | D
                Impedance : Impedance of input. Warning : Some device doesn't support de 50 Ohm coupling !

            Returns :
                self.GetAllErrors()
                or
                "-1" : Incorrect coupling
                "-2" : Incorrect impedance
        """
        if ACDC not in ["A", "D"]:
            return "-1"

        if Impedance not in self.__impedance__:
            return "-2"

        intimp = "50" if Impedance == 50 else "1M"
        self.__instr__.write(f"{self.__channel__}:CPL {ACDC}{intimp}")

        return self.__baseclass__.GetAllErrors()

    def SetOffset(self, Offset: float):
        """
        pySDS [Channel][SetOffset] : Configure the offset used.

            Arguments :
                Offset, in volts

            Returns :
                self.GetAllErrors()
        """

        self.__instr__.write(f"{self.__channel__}:OFST {Offset}V")
        return self.__baseclass__.GetAllErrors()

    def SetSkew(self, Skew: int):
        """
        pySDS [Channel][SetSkew] : Configure the delay between each channels, to compensate cable lenght matching

            Arguments :
                Skew : +- 100 ns to be applied

            Returns :
                self.GetAllErrors()
                or
                [1, -1] if Skew is not valid !
        """
        if Skew < 100 or Skew > 100:
            return [1, -1]  # Emulate error code !

        self.__instr__.write(f"{self.__channel__}:SKEW {Skew}")
        return self.__baseclass__.GetAllErrors()

    def EnableTrace(self):
        """
        pySDS [Channel][EnableTrace] : Enable the draw of the trace on the screen

            Arguments :
                None

            Return :
                self.GetAllErrors()
        """
        self.__instr__.write(f"{self.__channel__}:TRA ON")
        return self.__baseclass__.GetAllErrors()

    def DisableTrace(self):
        """
        pySDS [Channel][EnableTrace] : Disable the draw of the trace on the screen

            Arguments :
                None

            Return :
                self.GetAllErrors()
        """
        self.__instr__.query(f"{self.__channel__}:TRA OFF")
        return self.__baseclass__.GetAllErrors()

    def SetTraceUnit(self, Unit: str):
        """
        pySDS [Channel][SetTraceUnit] : Configure if the trace is on V (Volt) or A (Ampere)

            Arguments :
                Unit : V | A

            Returns :
                self.GetAllErrors()
        """

        self.__instr__.query(f"{self.__channel__}:UNIT {Unit}")
        return self.__baseclass__.GetAllErrors()

    def SetTraceDIV(self, Div: float):
        """
        pySDS [Channel][SetTraceVDIV] : Configure the gain on the unit of the trace, from 500E-6 to 10

            Arguments :
                Div : Gain to be applied

            Returns :
                self.GetAllErrors()
                or
                or -1 if invalid value
        """
        if Div < 0.000_5 or Div > 10:
            return -1

        self.__instr__.query(f"{self.__channel__}:VDIV {Div}")
        return self.__baseclass__.GetAllErrors()

    def EnableTraceInvert(self):
        """
        pySDS [Channel][EnableTraceInvert] : Enable invert the measure of the trace (* -1)

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.query(f"{self.__channel__}:INVS ON")
        return self.__baseclass__.GetAllErrors()

    def DisableTraceInvert(self):
        """
        pySDS [Channel][DisableTraceInvert] : Disable invert the measure of the trace (* -1)

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.query(f"{self.__channel__}:INVS OFF")
        return self.__baseclass__.GetAllErrors()



# ============================================================================================================
# SPI.py
# lheywang on 02/01/2025
#
# Advanced file for the trigger class, specialized on SPI bus
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase
from ..channel import SiglentChannel
from ..digital import SiglentDChannel


class SiglentSPI(SiglentBase):
    """
    pySDS [Trigger][SiglentSPI] :   Class herited from SiglentBase.
                                    Store all command related the control of the triggering system for the SPI bus

        Attributes :
            Herited from SiglentBase

        Methods :
            Private (0) :
                None

            Public (15):
                SetTriggerOnCLK :                   Set trigger on CLK Pin
                SetTriggerClockEdge :               Set trigger edge (act also on bus decoding)
                SetTriggerClockTimeout :            Set trigger timeout (act also on bus decoding)
                SetTriggerOnMOSI :                  Set trigger on MOSI Pin
                SetTriggerOnMISO :                  Set trigger on MISO Pin
                SetTriggerCSType :                  Set trigger Type (act also on bus decoding)
                SetTriggerOnCS :                    Set trigger on CS Pin
                SetTriggerOnNCS :                   Set trigger on NCS Pin
                ConfigureTriggerSource :            Set trigger Data Source
                ConfigureTriggerDataSequence :      Set trigger data Sequence
                ConfigureTriggerDataLen :           Set trigger Data Len
                SetTriggerBitOrder :                Set trigger bit order (act also on bus decoding)
    """

    def SetTriggerOnCLK(self, Channel, Threshold=1.65):
        """
        pySDS [SPI][SetTriggerOnCLK] : Configure the trigger on the SCLK pin.

            Arguments :
                Channel :       SiglentChannel or SiglentDCHannel related to the SCLK pin
                Threshold :     For analog channel only, the used voltage. Default to 1.65

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Channel
        """
        if type(Channel) is not SiglentChannel and type(Channel) is not SiglentDChannel:
            return [1, -1]

        cmd = f"TRSPI:CLK {Channel.__channel__}"

        if type(Channel) == SiglentChannel:
            cmd += f",{Threshold}"

        self.__instr__.write(cmd)
        return self.__baseclass__.GetAllErrors()

    def SetTriggerClockEdge(self, Edge: str):
        """
        pySDS [SPI][SetTriggerClockEdge] : Configure the edge of the clock the data is latched on

            Arguments :
                Edge : RISING | FALLING. The edge used.

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid edge
        """
        if Edge not in ["RISING", "FALLING"]:
            return [1, -1]

        self.__instr__.write(f"TRSPI:CLK:EDGE {Edge}")
        return self.__baseclass__.GetAllErrors()

    def SetTriggerClockTimeout(self, Timeout: str):
        """
        pySDS [SPI][SetTriggerClockTimeout] : Set the timeout value (related to clock) when the CS is timeout.

            Arguments :
                Timeout : from 100ns to 500ms (no check done)

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write(f"TRSPI:CLK:TIM {Timeout}")
        return self.__baseclass__.GetAllErrors()

    def SetTriggerOnMOSI(self, Channel, Threshold=1.65):
        """
        pySDS [SPI][SetTriggerOnMOSI] : Configure the trigger on the MOSI pin.

            Arguments :
                Channel :       SiglentChannel or SiglentDCHannel related to the MOSI pin
                Threshold :     For analog channel only, the used voltage. Default to 1.65

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Channel
        """
        if type(Channel) is not SiglentChannel and type(Channel) is not SiglentDChannel:
            return [1, -1]

        cmd = f"TRSPI:MOSI {Channel.__channel__}"

        if type(Channel) == SiglentChannel:
            cmd += f",{Threshold}"

        self.__instr__.write(cmd)
        return self.__baseclass__.GetAllErrors()

    def SetTriggerOnMISO(self, Channel, Threshold=1.65):
        """
        pySDS [SPI][SetTriggerOnMISO] : Configure the trigger on the MOSI pin

            Arguments :
                Channel :       SiglentChannel or SiglentDCHannel related to the MISO pin
                Threshold :     For analog channel only, the used voltage. Default to 1.65

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Channel
        """
        if type(Channel) is not SiglentChannel and type(Channel) is not SiglentDChannel:
            return [1, -1]

        cmd = f"TRSPI:MISO {Channel.__channel__}"

        if type(Channel) == SiglentChannel:
            cmd += f",{Threshold}"

        self.__instr__.write(cmd)
        return self.__baseclass__.GetAllErrors()

    def SetTriggerCSType(self, Type: str):
        """
        pySDS [SPI][SetTriggerCSType] : Set the CS type on the SPI bus

            Arguments :
                Type : CS | NCS | TIMEOUT

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid type
        """
        if Type not in ["CS", "NCS", "TIMEOUT"]:
            return [1, -1]

        self.__instr__.write(f"TRSPI:CSTP {Type}")

    def SetTriggerOnCS(self, Channel, Threshold=1.65):
        """
        pySDS [SPI][SetTriggerOnMISO] : Configure the trigger on the CS pin

            Arguments :
                Channel :       SiglentChannel or SiglentDCHannel related to the CS pin
                Threshold :     For analog channel only, the used voltage. Default to 1.65

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Channel
        """
        if type(Channel) is not SiglentChannel and type(Channel) is not SiglentDChannel:
            return [1, -1]

        cmd = f"TRSPI:CS {Channel.__channel__}"

        if type(Channel) == SiglentChannel:
            cmd += f",{Threshold}"

        self.__instr__.write(cmd)
        return self.__baseclass__.GetAllErrors()

    def SetTriggerOnNCS(self, Channel, Threshold=1.65):
        """
        pySDS [SPI][SetTriggerOnMISO] : Configure the trigger on the NCS pin

            Arguments :
                Channel :       SiglentChannel or SiglentDCHannel related to the NCS pin
                Threshold :     For analog channel only, the used voltage. Default to 1.65

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Channel
        """
        if type(Channel) is not SiglentChannel and type(Channel) is not SiglentDChannel:
            return [1, -1]

        cmd = f"TRSPI:NCS {Channel.__channel__}"

        if type(Channel) == SiglentChannel:
            cmd += f",{Threshold}"

        self.__instr__.write(cmd)
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerSource(self, Source: str):
        """
        pySDS [SPI][ConfigureTriggerSource] : Configure the trigger source for the SPI

            Arguments :
                Source : MOSI | MISO

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid source
        """
        if Source not in ["MOSI", "MISO"]:
            return [1, -1]

        self.__instr__.write(f"TRSPI:TRTY {Source}")
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerDataSequence(self, Sequence: list):
        """
        pySDS [SPI][ConfigureTriggerDataSequence] : Configure a matching bit sequence to trigger

            Arguments :
                Sequence : List of char / string : 0 | 1 | X to trigger. Len must match the datalen (not checked)

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid sequence (len not checked)
        """
        cmd = f"TRSPI:DATA "
        for char in Sequence:
            if char not in ["0", "1", "X"]:
                return [1, -1]
            cmd += f"{char},"
        cmd = cmd[:-1]  # removing last ","

        self.__instr__.write(cmd)
        return self.GetAllErrors()

    def ConfigureTriggerDataLen(self, Len: int):
        """
        pySDS [SPI][ConfigureTriggerDataSequence] : Configure a matching data lenght to trigger

            Arguments :
                Len : 4-96

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Len
        """
        if Len < 4 or Len > 96:
            return [1, -1]

        self.__instr__.write(f"TRSPI:DLEN {Len}")
        return self.__baseclass__.GetAllErrors()

    def SetTriggerBitOrder(self, Order: str):
        """
        pySDS [SPI][SetTriggerBitOrder] : Set the bit order on the SPI Bus

            Arguments :
                Order : MSB | LSB

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid order
        """
        if Order not in ["LSB", "MSB"]:
            return [1, -1]

        self.__instr__.write(f"TRSPI:BIT {Order}")
        return self.__baseclass__.GetAllErrors()



# ============================================================================================================
# Timebase.py
# lheywang on 21/12/2024
#
# Base file for the timebase class
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase


class SiglentTimebase(SiglentBase):
    """
    pySDS [Files][SiglentTimebase] :    Class herited from SiglentBase.
                                        Store all command related the control of timebase on the display

        Attributes :
            Herited from SiglentBase

        Methods :
            Private (0) :
                None

            Public (7):
    """

    def GetTimeDiv(self):
        """
        pySDS [Timebase][GetTimeDiv] : Return the time division used

            Arguments :
                None

            Returns :
                Time division used in seconds
        """
        return float(self.__instr__.query("TDIV?").strip().split(" ")[-1])

    def SetTimeDiv(self, Timebase: str):
        """
        pySDS [Timebase][SetTimeDiv] : Configure the timebase used

            Arguments :
                Timebase : Base to be used (from defined list right after)

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid timebase

            Possibles timebases :
                1NS 2NS 5NS
                10NS 20NS 50NS
                100NS 200NS 500NS
                1US 2US 5US
                10US 20US 50US
                100US 200US 500US
                1MS 2MS 5MS
                10MS 20MS 50MS
                100MS 200MS 500MS
                1S 2S 5S
                10S 20S 50S
                100S
        """
        if Timebase not in [
            "1NS",
            "2NS",
            "5NS",
            "10NS",
            "20NS",
            "50NS",
            "100NS",
            "200NS",
            "500NS",
            "1US",
            "2US",
            "5US",
            "10US",
            "20US",
            "50US",
            "100US",
            "200US",
            "500US",
            "1MS",
            "2MS",
            "5MS",
            "10MS",
            "20MS",
            "50MS",
            "100MS",
            "200MS",
            "500MS",
            "1S",
            "2S",
            "5S",
            "10S",
            "20S",
            "50S",
            "100S",
        ]:
            return [1, -1]
        self.__instr__.write(f"TDIV {Timebase}")
        return self.__baseclass__.GetAllErrors()

    def GetTriggerOffset(self):
        """
        pySDS [TimeBase][GetTriggerOffset] : Return the delay between trigger and center point

        - Pre-trigger acquisition — Data acquired before the trigger occurs. Negative trigger delays must be given in seconds.
        - Post-trigger acquisition — Data acquired after the trigger has occurred

            Arguments :
                None

            Returns :
                Time in seconds
        """
        return float(self.__instr__.query("TRDL?").strip().split(" ")[-1])

    def SetTriggerOffset(self, Offset):
        """
        pySDS [TimeBase][SetTriggerOffset] : Offset the delay between trigger and center point.

        - Pre-trigger acquisition — Data acquired before the trigger occurs. Negative trigger delays must be given in seconds.
        - Post-trigger acquisition — Data acquired after the trigger has occurred

            Arguments :
                Offset : Delay in seconds to offset

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write(f"TRDL {Offset}S")
        return self.__baseclass__.GetAllErrors()

    def SetMagnifierZoom(self, Zoom):
        """
        pySDS [TimeBase][SetMagnifierZoom] : Configure the zoom on the screen.

            Arguments :
                Zoom : Two options (depending on the device !) :
                    - Time with units, from 1NS to current timebase (only checked globally, not for actual settings) ==> Pass an str type
                    - Factor, from 1 to 2 000 000 (Older !) ==> Pass a non str type

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid zoom

            Possibles timebases :
                1NS 2NS 5NS
                10NS 20NS 50NS
                100NS 200NS 500NS
                1US 2US 5US
                10US 20US 50US
                100US 200US 500US
                1MS 2MS 5MS
                10MS 20MS 50MS
                100MS 200MS 500MS
                1S 2S 5S
                10S 20S 50S
                100S
        """
        # Type 1, which is OLD !
        if type(Zoom) == str:
            if Zoom not in [
                "1NS",
                "2NS",
                "5NS",
                "10NS",
                "20NS",
                "50NS",
                "100NS",
                "200NS",
                "500NS",
                "1US",
                "2US",
                "5US",
                "10US",
                "20US",
                "50US",
                "100US",
                "200US",
                "500US",
                "1MS",
                "2MS",
                "5MS",
                "10MS",
                "20MS",
                "50MS",
                "100MS",
                "200MS",
                "500MS",
                "1S",
                "2S",
                "5S",
                "10S",
                "20S",
                "50S",
                "100S",
            ]:
                return [1, -1]
        else:
            if Zoom < 1 or Zoom > 2_000_000:
                return [1, -1]

        self.__instr__.write(f"HMAG {Zoom}")
        return self.__baseclass__.GetAllErrors()

    def SetMagnifierPosition(self, Position):
        """
        pySDS [TimeBase][SetMagnifierPosition] : Place the zoom position

            Arguments :
                Two options (depending on the device) :
                    - Time position, with units. ==> Pass an str type
                    - Div factor (older !) ==> Pass an non str type

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Position
        """
        if type(Position) == str:
            if Position not in [
                "1NS",
                "2NS",
                "5NS",
                "10NS",
                "20NS",
                "50NS",
                "100NS",
                "200NS",
                "500NS",
                "1US",
                "2US",
                "5US",
                "10US",
                "20US",
                "50US",
                "100US",
                "200US",
                "500US",
                "1MS",
                "2MS",
                "5MS",
                "10MS",
                "20MS",
                "50MS",
                "100MS",
                "200MS",
                "500MS",
                "1S",
                "2S",
                "5S",
                "10S",
                "20S",
                "50S",
                "100S",
            ]:
                return [1, -1]

        else:
            # No conditions specified, so what to do ?
            pass

        self.__instr__.write(f"HPOS {Position}")
        return self.__baseclass__.GetAllErrors()



# ============================================================================================================
# Generics.py
# lheywang on 21/12/2024
#
# Base file for the generics SCPI class.
# This class could easily be reused with any IEEE 488.1 compliant device.
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase


class SCPIGenerics(SiglentBase):
    def ClearStatus(self):
        """
        PySDS [ClearStatus] :   Clear the status register

            Arguments :
                None

            Returns :
                None
        """

        self.__instr__.write("*CLS")
        return

    def ReadCMR(self):
        """
        PySDS [ReadCMR] :   Read and clear the CMR register

            Arguments :
                None

            Returns :
                Integer : Register value
        """

        Ret = self.__instr__.query("CMR?")
        return int(Ret.strip().split(" ")[-1])

    def ReadDDR(self):
        """
        PySDS [ReadDDR] :   Read and clear the DDR register

            Arguments :
                None

            Returns :
                Integer : Register value
        """

        Ret = self.__instr__.query("DDR?")
        return int(Ret.strip().split(" ")[-1])

    def ReadESE(self):
        """
        PySDS [ReadESE] :   Read and clear the ESE register

            Arguments :
                None

            Returns :
                Integer : Register value
        """

        Ret = self.__instr__.query("*ESE?")
        return int(Ret.strip())

    def ReadESR(self):
        """
        PySDS [ReadESR] :   Read and clear the ESE register

            Arguments :
                None

            Returns :
                Integer : Register value
        """

        Ret = self.__instr__.query("*ESR?")
        return int(Ret.strip())

    def ReadEXR(self):
        """
        PySDS [ReadEXR] :   Read and clear the EXR register

            Arguments :
                None

            Returns :
                Integer : Register value
        """

        Ret = self.__instr__.query("EXR?")
        return int(Ret.strip().split(" ")[-1])

    def ReadIDN(self):
        """
        PySDS [ReadIDN] :   Read back the device name

            Arguments :
                None

            Returns :
                String : The output of the command
        """

        return self.__instr__.query("*IDN?").strip()

    def ReadINR(self):
        """
        PySDS [ReadINR] :   Read and clear the device status

            Arguments :
                None

            Returns :
                Integer : Register value
        """

        return int(self.__instr__.query("INR?").strip().split(" ")[-1])

    def ReadOPC(self):
        """
        PySDS [ReadOPC] :   Read the Operation Complete status bit.
                            Actually, this function always return 1, because the device respond when the operation is complete...

            Arguments :
                None

            Returns :
                Integer : Register value
        """

        return int(self.__instr__.query("*OPC?").strip())

    def ReadOPT(self):
        """
        PySDS [ReadOPT] :   Read the installed options on the device

            Arguments :
                None

            Returns :
                String : The output of the command
        """

        return self.__instr__.query("*OPT?").strip()

    def ReadSRE(self):
        """
        PySDS [ReadSRE] :   Read the service request enable register value

            Arguments :
                None

            Returns :
                Integer : Register value
        """

        return int(self.__instr__.query("*SRE?").strip())

    def ReadSTB(self):
        """
        PySDS [ReadSTB] :   Read the status register

            Arguments :
                None

            Returns :
                Integer : Register value
        """

        return int(self.__instr__.query("*STB?").strip())

    def SetESE(self, value: int):
        """
        PySDS [SetESE] :   Write the ESE Register

            Arguments :
                Integer : Value to be written

            Returns :
                self.GetAllErrors() returns (List of errors)
        """

        if value > 255 or value < 0:
            return -1

        self.__instr__.write(f"*ESE {value}")
        return self.__baseclass__.GetAllErrors()

    def SetESR(self, value: int):
        """
        PySDS [SetESR] :   Write the ESR Register

            Arguments :
                Integer : Value to be written

            Returns :
                self.GetAllErrors() returns (List of errors)
        """

        if value > 128 or value < 0:
            return -1

        self.__instr__.write(f"*ESR {value}")
        return self.__baseclass__.GetAllErrors()

    def SetOPC(self):
        """
        PySDS [SetOPC] :   Write the OPC (Operation Complete) Status bit

            Arguments :
                None

            Returns :
                self.GetAllErrors() returns (List of errors)
        """
        self.__instr__.write("*OPC")
        return self.__baseclass__.GetAllErrors()

    def SetSRE(self, value: int):
        """
        PySDS [SetSRE] :   Write the ESR Register (Service Request Enable Register)

            Arguments :
                Integer : Value to be written

            Returns :
                self.GetAllErrors() returns (List of errors)
        """

        if value > 256 or value < 0:
            return -1

        self.__instr__.write(f"*SRE {value}")
        return self.__baseclass__.GetAllErrors()



# ============================================================================================================
# Files.py
# lheywang on 19/12/2024
#
# Base file for the file management class
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase


class SiglentFiles(SiglentBase):
    """
    pySDS [Files][SiglentFiles] :   Class herited from SiglentBase.
                                    Store all command related the control of filesystem.

        Attributes :
            Herited from SiglentBase

        Methods :
            Private (0) :
                None

            Public (7):
                CaptureBMPScreen :          Capture the screen as a BMP File
    """

    def CaptureBMPScreen(self, File):
        """
        pySDS [Files][CaptureBMPScreen] : Capture the screen and write a BMP file

            Arguments :
                File : Path to be written

            Returns :
                self.GetAllErrors()
        """

        self.__instr__.write("SCDP")
        data = self.__instr__.read_raw()

        with open(File, "wb") as f:
            f.write(data)

        return self.__baseclass__.GetAllErrors()



# ============================================================================================================
# PySDS.py
# lheywang on 17/12/2024
#
# Base file for the whole package class
#
# ============================================================================================================

# Python libraries
import ipaddress
import tomllib
from datetime import datetime
from warnings import warn

# Poetry managed lbraries
import pyvisa  # type: ignore

# Others files
from . import SiglentAcquisition
from . import SiglentChannel
from . import SiglentCommunication
from . import SiglentCursor
from . import SiglentDecode
from . import ACAL, DATE, COUNTER
from . import SiglentDigital
from . import SiglentScreen
from . import SiglentFiles
from . import SCPIGenerics
from . import SiglentHistory
from . import SiglentMaths
from . import SiglentMeasure
from . import SiglentPassFail
from . import SiglentReference
from . import SiglentTimebase
from . import SiglentTrigger
from . import SiglentWaveform


class PySDS:
    """
    PySDS [class] : Parent class of the PySDS package.
                    Handle actually all of basic SCPI commands, and call subclasses for some advanced functionnalities !

        Attributes :
            Private :
                __ip__ :            ip of the device. Used internally to check it's validity.
                __rm__ :            pyvisa ressource manager
                __instr__ :         Handle to the pyvisa object to interract with the device
                __ConfigFile__ :    Configuration file used for the scope.
                __Config__ :        Parsed configuration toml file
                __Generics__ :      SCPIGenerics class. Used for low level interraction with the device.

            Public :
                ** Standard variables **
                DeviceOpenned :     Get a non 0 value if the device was openned correctly. Otherwise, take 0
                model :             Device model, parsed from *IDN command
                SN :                Device SN, parsed from *IDN command
                Firmware :          Device firmware revision, parsed from *IDN command

                ** SCPI Variables **
                Channel :           A list of channel presents on the device. Each is it's own SiglentChannel class.
                Trigger :           A SiglentTrigger class



        Methods :
            Private :

            Public :

        Parents :
            None

        Subclass :
            None

    """

    def __init__(self, IP: str):
        """
        PySDS [init] :  Initialize the class.
                        Use some configuration file to initialize properly the oscilloscope, and read it's actual state to make sure to fetch the real state
                        May take some time since a lot of network requests are done here !

            Arguments :
                IP : A string IP address, version 4 of where the ressource shall be allocated

            Returns :
                None
        """

        # First, validate the IP and try to open the ressource !
        try:
            self.__ip__ = ipaddress.ip_address(IP)
        except ValueError:
            print(
                "     [ PySDS ] [ Init ] : Incorrect IP was passed to the constructor"
            )
            self.DeviceOpenned = 0
            return

        try:
            self.__rm__ = pyvisa.ResourceManager()
            self.__instr__ = self.__rm__.open_resource(f"TCPIP0::{IP}::inst0::INSTR")
        except Exception as e:
            print(
                "     [ PySDS ] [ Init ] : Unable to access to the device. Check if the IP is right, or if you can ping it !"
            )
            print(f"                           Error code : {e}")
            self.DeviceOpenned = 0
            return

        # Then, request for the IDN command.
        # Typical return : Siglent Technologies,SDS824X HD,SDS08A0C802019,3.8.12.1.1.3.8
        IDN = self.__instr__.query("*IDN?")
        IDN = IDN.split(",")

        # Check if the brand is the right one, or this library isn't going to work !
        if IDN[0].find("Siglent") == -1:
            print("     [ PySDS ] [ Init ] : Found a non Siglent Device on this IP !")
            self.DeviceOpenned = 0
            return

        # Parse some different fields
        self.model = IDN[1]
        self.SN = IDN[2]
        self.Firmware = IDN[3]

        # Load the right configuration file
        # First, replace any space in the name with a "-" to ensure compatibility within different OS
        self.model = self.model.replace(" ", "-")

        # Load the right configuration file without the SDS in front
        self.__ConfigFile__ = self.model[3:] + ".toml"

        self.__Config__ = None
        with open(f"config/{self.__ConfigFile__}", "rb") as f:
            self.__Config__ = tomllib.load(f)

        # Create a generic class, for internal usage only
        self.__Generics__ = SCPIGenerics(self.__instr__, self)

        # Now, initialize some parameters from the configuration file
        self.Channel = []
        for index in range(self.__Config__["Specs"]["Channel"]):
            self.Channel.append(
                SiglentChannel(
                    self.__instr__,
                    self,
                    f"C{index}",
                    self.__Config__["Specs"]["impedance"],
                )
            )

        # Then, initialize all of the subclass
        # Warning : Some fetures may not be available globally, but due to minor variations they're here for all.

        self.Acquistion = SiglentAcquisition(self.__instr__, self)
        self.Autotest = SiglentPassFail(self.__instr__, self)
        self.Communication = SiglentCommunication(self.__instr__, self)
        self.Cursor = SiglentCursor(self.__instr__, self)
        self.Decode = SiglentDecode(self.__instr__, self)
        self.Digital = SiglentDigital(self.__instr__, self)
        self.Display = SiglentScreen(self.__instr__, self)
        self.Files = SiglentFiles(self.__instr__, self)
        self.History = SiglentHistory(self.__instr__, self)
        self.Maths = SiglentMaths(self.__instr__, self)
        self.Measure = SiglentMeasure(self.__instr__, self)
        self.Reference = SiglentReference(self.__instr__, self)
        self.Trigger = SiglentTrigger(self.__instr__, self)
        self.Timebase = SiglentTimebase(self.__instr__, self)
        self.Waveform = SiglentWaveform(self.__instr__, self)

        # For some older device, load additionnal commands that are depecrated in the newest models / firmwares
        if "ACAL" in self.__Config__["Specs"]["LegacyFunctions"]:
            self.Calibration = ACAL(self.__instr__, self)

        if "DATE" in self.__Config__["Specs"]["LegacyFunctions"]:
            self.Date = DATE(self.__instr__, self)

        if "COUNTER" in self.__Config__["Specs"]["LegacyFunctions"]:
            self.Counter = COUNTER(self.__instr__, self)

        self.DeviceOpenned = 1
        return

    def __repr__(self):
        """
        PySDS [repr] :  Basic print of the connected device.
                        Aimed to the developper, and thus expose more informations than the __str__ function !

            Arguments :
                None

            Returns :
                None
        """

        print(f"Device on {self.__ip__} : \nType : {self.model} ")
        return

    def __str__(self):
        """
        PySDS [repr] :  Basic print of the connected device.
                        Aimed to the user, and thus expose less informations than the __repr__ function !

            Arguments :
                None

            Returns :
                None
        """

        print(f"Device on {self.__ip__} : \nType : {self.model} ")
        return

    #
    #   STATUS
    #

    def GetAllStatus(self):
        """
        PySDS [GetAllStatus] :  Return the status of the STB, ESR, INR, DDR, CMD, EXR and URR Registers.

            Arguments :
                None

            Returns :
                List of integers with the values in order
        """

        # Querry
        Ret = self.__instr__.query("ALST?")

        # Split comma. Format : ALST STB, Val, ESR..
        # Get only the usefull values
        Ret = Ret.strip().split(",")
        return [
            int(Ret[1]),
            int(Ret[3]),
            int(Ret[5]),
            int(Ret[7]),
            int(Ret[9]),
            int(Ret[11]),
            int(Ret[13]),
        ]

    #
    #   BUZZER
    #

    def EnableBuzzer(self):
        """
        PySDS [EnableBuzzer] :  Enable the device buzzer

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("BUZZ ON")
        return self.GetAllErrors()

    def DisableBuzzer(self):
        """
        PySDS [DisableBuzzer] : Disable the device buzzer

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("BUZZ OFF")
        return self.GetAllErrors()

    def GetBuzzerEnablingState(self):
        """
        PySDS [GetBuzzerEnablingState] :    Return the buzzer enabling state (ON or OFF)

        Arguments :
            None

        Returns :
            True | False
        """
        Ret = self.__instr__.query("BUZZ?").strip().split(" ")[-1]
        if Ret == "ON":
            return True
        return False

    #
    #   CALIBRATION
    #

    def Calibrate(self):
        """
        PySDS [Calibrate] : Calibrate the device.
                            This is actually the fast one, which does not do a full analog frontend calibration.

        WARNING :   Leaving probes and other elements connected may affect the result.
                    Make sure to calibrate the device in proper conditions !

        Arguments :
            None

        Returns :
            Integer : If 0, then calibration was sucessfull.
        """

        Ret = self.__instr__.query("*CAL?")
        return int(Ret.strip().split(" ")[-1])

    #
    #   AUTOSET
    #

    def Autoset(self):
        """
        pySDS [Autoset] : Launch an autoset procedure

        WARNING :   This should be avoided as possible, since the autoset isn't the most reliable thing in the world.
                    It's possible that the scope will show a detail of the waweform where there is some others things to be seen.
                    Use in a maximal number of case the manual settings to precisely control the predicted signal

            Arguments :
                None

            Returns :
                self.GetAllErrors() : List of errors that occured
        """
        self.__instr__.write("ASET")
        return self.GetAllErrors()

    #
    #   STANDARD SCPI COMMANDS
    #

    # =============================================================================================================================================
    """
    Up to this point, all functions shall be working on any device, even other than Siglent ones since they're part
    of the IEEE 488.1 specification.

    In any way, the class can't be constructed without a compatible device, that's why I didn't create a global SCPI engine...
    """
    # =============================================================================================================================================

    def LockDevicePanel(self):
        """
        PySDS [LockDevicePanel] : Lock the device front panel to prevent any actions of the user

        WARNING : This command seems to exhibit some weird response and no action at all on an SDS824X-HD

            Arguments :
                None

            Returns :
                self.GetAllErrors() : List of errors
        """
        self.__instr__.write("LOCK ON")
        return self.GetAllErrors()

    def UnlockDevicePanel(self):
        """
        PySDS [UnlockDevicePanel] : Unlock the device front panel to enable any actions of the user

        WARNING : This command seems to exhibit some weird response and no action at all on an SDS824X-HD

            Arguments :
                None

            Returns :
                self.GetAllErrors() : List of errors
        """
        self.__instr__.write("LOCK OFF")
        return self.GetAllErrors()

    def GetDevicePanelLockState(self):
        """
        PySDS [GetDevicePanelLockState] : Return the status of the lock on the front panel

        WARNING : This command seems to exhibit some weird response and no action at all on an SDS824X-HD

            Arguments :
                None

            Returns :
                Boolean : Lock (True) or not (False)
        """
        Ret = self.__instr__.query("LOCK?").strip().split(" ")[-1]
        if Ret == "ON":
            return True
        return False

    def RecallPreset(self, PresetNumber: int):
        """
        PySDS [RecallPreset] :  Apply a previously stored list of settings on the device.
                                Can only be called after the call of SavePreset function !
                                If 0 is passed, this is the default config.

            Argument :
                PresentNumber : Integer of the position to store the preset

            Returns :
                self.GetAllErrors() returns (List of errors)
                or
                -1 : Invalid preset ID
        """
        if PresetNumber > 20 or PresetNumber < 0:
            return [1, -1]

        self.__instr__.write(f"*RCL {PresetNumber}")
        return self.GetAllErrors()

    def SavePresent(self, PresetNumber: int):
        """
        PySDS [SavePresent] :   Store the settings of the device into a defined non volatile memory location.
                                Number 0 is not valid, since this location is the default preset.

            Argument :
                PresentNumber : Integer of the position to store the preset

            Returns :
                self.GetAllErrors() returns (List of errors)
                or
                -1 : Invalid preset ID
        """
        if PresetNumber > 20 or PresetNumber < 1:
            return [1, -1]

        self.__instr__.write(f"*SAV {PresetNumber}")
        return self.GetAllErrors()

    def ResetDevice(self):
        """
        PySDS [ResetDevice] : Perform a software reset of the device

        Arguments :
            None

        Returns :
            self.GetAllErrors() returns (List of errors)
        """

        self.__instr__.write("*RST")
        return self.GetAllErrors()

    # =============================================================================================================================================
    """
    Now, let's define some more advanced functions that will call some previously defined ones.

    It's more aimed at the user, even if the previous ones remains accessibles, since theses functions will provide more content.
    
    """
    # =============================================================================================================================================

    def GetAllErrors(self, toprint=False):
        """
        PySDS [GetAllErrors] :  Read the device errors, and until at least one error exist, continue to read it.
                                For each errors, it will be printed in console and returned on a list, with it's lengh in first position.

                                This function also trigger a reading of the status of the device to detect if value where adapted or cancelled.

            Arguments :
                toprint : Shall we print the decoded output on the console ? Default to false.

            Returns :
                List :
                    Index 0 :       Number of errors that occured
                    Index 1 - n :   Device errors codes
        """

        FetchNextError = True
        Errors = []
        Errors.append(0)

        # For each loop, we ask the device an error
        # If not 0, then we parse it and add it to the list
        # When the last error has been fetched (or no errors at all !), we exit the loop

        while FetchNextError:
            Ret = self.__Generics__.ReadEXR()

            if Ret == 0:
                FetchNextError = False

            else:
                Errors[0] += 1
                Errors.append(int(Ret))

                if toprint == True:
                    # Theses errors messages came from the Siglent SCPI documentation, and are only here to help the developper to get the error easily !
                    match Ret:
                        case 21:
                            print(
                                f"     [ PySDS ] [ GetAllErrors ] : ({Ret}) Permission error. The command cannot be executed in local mode."
                            )
                        case 22:
                            print(
                                f"     [ PySDS ] [ GetAllErrors ] : ({Ret}) Environment error. The instrument is not configured to correctly process command. For instance, the oscilloscope cannot be set to RIS at a slow timebase."
                            )
                        case 23:
                            print(
                                f"     [ PySDS ] [ GetAllErrors ] : ({Ret}) Option error. The command applies to an option which has not been installed."
                            )
                        case 25:
                            print(
                                f"     [ PySDS ] [ GetAllErrors ] : ({Ret}) Parameter error. Too many parameters specified."
                            )
                        case 26:
                            print(
                                f"     [ PySDS ] [ GetAllErrors ] : ({Ret}) Non-implemented command."
                            )
                        case 32:
                            print(
                                f"     [ PySDS ] [ GetAllErrors ] : ({Ret}) Waveform descriptor error. An invalid waveform descriptor has been detected."
                            )
                        case 36:
                            print(
                                f"     [ PySDS ] [ GetAllErrors ] : ({Ret}) Panel setup error. An invalid panel setup data block has been detected."
                            )
                        case 50:
                            print(
                                f"     [ PySDS ] [ GetAllErrors ] : ({Ret}) No mass storage present when user attempted to access it."
                            )
                        case 53:
                            print(
                                f"     [ PySDS ] [ GetAllErrors ] : ({Ret}) Mass storage was write protected when user attempted to create, or a file, to delete a file, or to format the device."
                            )
                        case 58:
                            print(
                                f"     [ PySDS ] [ GetAllErrors ] : ({Ret}) Mass storage file not found."
                            )
                        case 59:
                            print(
                                f"     [ PySDS ] [ GetAllErrors ] : ({Ret}) Requested directory not found."
                            )
                        case 61:
                            print(
                                f"     [ PySDS ] [ GetAllErrors ] : ({Ret}) Mass storage filename not DOS compatible, or illegal filename."
                            )
                        case 62:
                            print(
                                f"     [ PySDS ] [ GetAllErrors ] : ({Ret}) Cannot write on mass storage because filename already exists."
                            )

        # When the loop exist, we return the list
        Retval = self.GetDeviceStatus(toprint)

        if Retval != 0:
            Errors[0] += 1

            tmp = 0
            power = 0
            for bit in Retval:
                tmp += 2**power * bit
                power += 1

            Errors.append(tmp + 1000)
            # Increment of 1000 to signify an error in the MSB register

        return Errors

    def GetDeviceStatus(self, toprint=False):
        """
        PySDS [GetDeviceStatus] :   Get the device status, and parse it to make it easier to use for developpers or users.
                                    Print each status bit

            Argument :
                toprint : Shall we print the decoded output on the console ? Default to false.

            Returns :
                List of lenght 16, for each bit
        """

        # Fetch the value
        Ret = self.__Generics__.ReadINR()

        # Mask each bit in the range.
        # We do this by logic AND and shifting to get back to 0 | 1
        Bits = []
        for power in range(16):
            Bits.append((Ret & pow(2, power)) >> power)

        print("Device status :")
        print("Bit | Status | Message")
        for index, bit in enumerate(Bits):
            match index:
                case 0:
                    message = "A new signal has been acquired"
                case 1:
                    message = "A screen dump has terminated"
                case 2:
                    message = "A return to the local state is detected"
                case 3:
                    message = "A time-out has occurred in a data block transfer"
                case 4:
                    message = "A segment of a sequence waveform has been acquired"
                case 5:
                    message = "Reserved for LeCroy use"
                case 6:
                    message = 'Memory card, floppy or hard disk has become full in "AutoStore Fill" mode'
                case 7:
                    message = (
                        "A memory card, floppy or hard disk exchange has been detected"
                    )
                case 8:
                    message = "Waveform processing has terminated in Trace A"
                case 9:
                    message = "Waveform processing has terminated in Trace B"
                case 10:
                    message = "Waveform processing has terminated in Trace C"
                case 11:
                    message = "Waveform processing has terminated in Trace D"
                case 12:
                    message = "Pass/Fail test detected desired outcome"
                case 13:
                    message = "Trigger is ready"
                case 14:
                    message = "Reserved for future use"
                case 15:
                    message = "Reserved for future use"

            if toprint == True:
                if bit == 1:
                    print(f" {index:2} |  {bit:5} | {message}")
                else:
                    print(f" {index:2} |  {bit:5} | -")

        return Bits

    def GetDeviceOptions(self):
        """
        PySDS [GetDeviceOptions] :  Return the list of the installed device options.
                                    Function isn't working for now, but the response seems correct.
                                    --> Return 0 where it shall return OPC 0...

            Arguments :
                None

            Returns :
                List of String for all options
        """

        Ret = self.__Generics__.ReadOPT()
        return Ret.split(" ")[-1].split(",")

    def GetDeviceStatus2(self):
        """
        PySDS [GetDeviceStatus2] :  Read the device status, and parse it to be easier for the user to read !

            Arguments :
                None

            Returns :
                List of lenght 16, for each bit
        """

        # Fetch the value
        Ret = self.__Generics__.ReadSTB()

        # Mask each bit in the range.
        # We do this by logic AND and shifting to get back to 0 | 1
        Bits = []
        for power in range(8):
            Bits.append((Ret & pow(2, power)) >> power)

        print("Device status register :")
        print("Bit | Status | Message")
        for index, bit in enumerate(Bits):
            match index:
                case 0:
                    message = "An enabled Internal state change has occurred"
                case 1:
                    message = "Reserved"
                case 2:
                    message = "A command data value has been adapted"
                case 3:
                    message = "Reserved"
                case 4:
                    message = "Output queue is not empty "
                case 5:
                    message = "An ESR enabled event has occurred"
                case 6:
                    message = "At least 1 bit in STB masked by SRE is one service is requested"
                case 7:
                    message = "Reserved for future use"

            if bit == 1:
                print(f" {index:2} |  {bit:5} | {message}")
            else:
                print(f" {index:2} |  {bit:5} | -")

        return Bits



# ============================================================================================================
# CAN.py
# lheywang on 02/01/2025
#
# Advanced file for the trigger class, specialized on CAN bus
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase
from ..channel import SiglentChannel
from ..digital import SiglentDChannel


class SiglentCAN(SiglentBase):
    """
    pySDS [Trigger][SiglentCAN] :   Class herited from SiglentBase.
                                    Store all command related the control of the triggering system for the CAN bus

        Attributes :
            Herited from SiglentBase

        Methods :
            Private (0) :
                None

            Public (7):
                SetTriggerOnCANH :          Set trigger Pin.
                SetTriggerOnCondition :     Set trigger on a condition
                ConfigureTriggerID :        Set trigger ID
                ConfigureTriggerIDLen :     Set trigger ID Len
                ConfigureTriggerData1 :     Set trigger first byte
                ConfigureTriggerData2 :     Set trigger second byte
                SetTriggerBaud :            Set trigger baud

    """

    def SetTriggerOnCANH(self, Channel, Threshold=1.65):
        """
        pySDS [CAN][SetTriggerOnCANH] : Configure the trigger on the CANH pin

            Arguments :
                Channel :       SiglentChannel or SiglentDCHannel related to the CANH pin
                Threshold :     For analog channel only, the used voltage. Default to 1.65

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Channel
        """
        if type(Channel) is not SiglentChannel and type(Channel) is not SiglentDChannel:
            return [1, -1]

        cmd = f"TRCAN:CANH {Channel.__channel__}"

        if type(Channel) == SiglentChannel:
            cmd += f",{Threshold}"

        self.__instr__.write(cmd)
        return self.__baseclass__.GetAllErrors()

    def SetTriggerOnCondition(self, Condition: str):
        """
        pySDS [CAN][SetTriggerOnCondition] : Configure the trigger on a condition on the CAN Bus

            Arguments :
                Condition : START | REMOTE | ID | ID_AND_DATA | ERROR

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid condition

            Possibles conditions
                START— Start condition.
                REMOTE— Remote frame
                ID— Specifies a search based on ID bits and ID.
                ID_AND_DATA— Specify a search based on ID bits, ID and data.
                ERROR— Error frame
        """
        if Condition not in ["START", "REMOTE", "ID", "ID_AND_DATA", "ERROR"]:
            return [1, -1]

        self.__instr__.write(f"TRCAN:CON {Condition}")
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerID(self, ID: int):
        """
        pySDS [CAN][ConfigureTriggerID] : Set the ID for ID and ID_AND_DATA mode of trigger

            Arguments :
                ID : Integer between 0-2048 (11 bits) or 0-536870912 (29 bits)

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write(f"TRCAN:ID {ID}")
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerIDLen(self, Len: str):
        """
        pySDS [CAN][ConfigureTriggerID] : Set the ID len for ID and ID_AND_DATA mode of trigger.

            Arguments :
                Len : 11BITS | 29BITS

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Len
        """
        if Len not in ["11BITS", "29BITS"]:
            return [1, -1]

        self.__instr__.write(f"TRCAN:IDL {Len}")
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerData1(self, Value: int):
        """
        pySDS [CAN][ConfigureTriggerData1] : Configure the data (first byte) to be used to trigger for ID_AND_DATA mode.

            Arguments :
                Value: 0-255 value

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid value
        """
        if Value < 0 or Value > 255:
            return [1, -1]

        self.__instr__.write(f"TRCAN:DATA {Value}")
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerData2(self, Value: int):
        """
        pySDS [CAN][ConfigureTriggerData2] : Configure the data (second byte) to be used to trigger for ID_AND_DATA mode.

            Arguments :
                Value: 0-255 value

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid value
        """
        if Value < 0 or Value > 255:
            return [1, -1]

        self.__instr__.write(f"TRCAN:DAT2 {Value}")
        return self.__baseclass__.GetAllErrors()

    def SetTriggerBaud(self, Baud):
        """
        pySDS [CAN][SetTriggerBaud] : Set the baud rate of the UART comm

            Arguments :
                Baud : Value. (As a string for standard values, or as an integer for non standard)*

                * Sending a standard as integer will be seen as custom but will also works fine !

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Baud

            Standard baud values :
                5k
                10k
                20k
                50k
                100k
                125k
                250k
                500k
                800k
                1M
        """
        if Baud < 5_000 or Baud > 1_000_000:
            return [1, -1]

        if Baud not in [
            "5k",
            "10k",
            "20k",
            "59k",
            "100k",
            "125k",
            "250k",
            "500k",
            "800k",
            "1M",
        ]:
            cmd = f"TRCAN:BAUD CUSTOM,{Baud}"
        else:
            cmd = f"TRCAN:BAUD {Baud}"

        self.__instr__.write(cmd)
        return self.__baseclass__.GetAllErrors()



# ============================================================================================================
# Cursor.py
# lheywang on 19/12/2024
#
# Base file for the cursor class
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase
from datetime import datetime


class DATE(SiglentBase):
    pass

    def GetDate(self):
        """
        PySDS [GetDate] :   Read and return the date stored on the oscilloscope RTC

        Actually, this function does not work, despite that it's presence is stated on the datasheet.
        --> Possible issues :
                Function non implemented ?
                Syntax not OK ?

            Arguments :
                None

            Returns :
                Python Datetime object
        """

        Ret = self.__instr__.query("DATE?")
        Ret = Ret.strip().split(" ")[1:]

        # Why did they express month like that ? Cannot they send the number ?
        match Ret[1]:
            case "JAN":
                month = 1
            case "FEB":
                month = 2
            case "MAR":
                month = 3
            case "APR":
                month = 4
            case "MAY":
                month = 5
            case "JUN":
                month = 6
            case "JUL":
                month = 7
            case "AUG":
                month = 8
            case "SEP":
                month = 9
            case "OCT":
                month = 10
            case "NOV":
                month = 11
            case "DEC":
                month = 12

        return datetime(Ret[2], month, Ret[0], Ret[3], Ret[4], Ret[5])

    def SetDate(self, Date: datetime):
        """
        PySDS [SetDate] :   Set the internal RTC date and time

            Arguments :
                Python Datetime object

            Returns :
                self.GetAllErrors() returns (List of errors)
        """
        self.__instr__.write(
            f"DATE {Date.day},{Date.strftime("%b").upper()},{Date.year},{Date.hour},{Date.minute},{Date.second}"
        )
        return self.__baseclass__.GetAllErrors()



# ============================================================================================================
# Cursor.py
# lheywang on 19/12/2024
#
# Base file for the cursor class
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase
from ..channel import SiglentChannel
from warnings import warn


class SiglentCursor(SiglentBase):
    """
    pySDS [Cursor][SiglentCursor] : Class herited from SiglentBase.
                                    Store all command related the control of cursors
        Attributes :
            Herited from SiglentBase

        Methods :
            Private (0) :
                None

            Public (6):
                SetCursorMode :     Set the mode for a cursor
                GetCursorMode :     Read the mode of a cursor
                PlaceCursor :       Place a cursor
                GetPlacedCursor :   Return the placed cursor on a trace
                SetCursorType :     Configure the cursor type
                GetCursorValue :    Read the value of a cursor
    """

    def SetCursorMode(self, Mode):
        """
        pySDS [Cursor][SetCursorMode] : Set the mode of operation of the cursors

            Arguments :
                Mode :  The mode wanted between OFF | MANUAL | TRACK | (ON)
                        *ON is reserved to some legacy devices, it's usage will trigger a warning !

            Returns :
                self.GetAllErrors()
                or
                -1 if invalid mode has been passed
        """
        if Mode not in ["ON", "OFF", "TRACK", "MANUAL"]:
            return [1, -1]

        if Mode == "ON":
            warn(
                "      pySDS [Cursor][SetCursorMode] : Usage of a legacy mode has been done ! Check if it's correct !"
            )

        self.__instr__.write(f"CRMS {Mode}")
        return self.__baseclass__.GetAllErrors()

    def GetCursorMode(self):
        """
        pySDS [Cursor][GetCursorMode] : Return the mode of operation of the cursor

            Arguments :
                None

            Returns :
                Device response
        """
        return self.__instr__.query("CRMS?").strip().split(" ")[-1]

    def PlaceCursor(self, Channel: SiglentChannel, Cursor, Position):
        """
        pySDS [Cursor][PlaceCursor] : Place a cursor

        WARNING :   Each cursor is unique, you can't set twice the cursor on two different channels.
                    The device will consider the last call only.

        WARNING2 :  Some settings may trigger an out of bound error. Make sure to check the return code of this function.

            Arguments :
                Channel : The channel to which the cursor belong. This is a SiglentChannel class.
                Cursor : VREF | VDIF | TREF | TDIF | HREF | HDIF
                Position : Value where to place the cursor. This value is sensitive to errors and isn't check internally

            Returns :
                GetAllErrors() : Read and return all of the device errors
        """
        if Cursor not in ["VREF", "VDIF", "TDIF", "TREF", "HREF", "HDIF"]:
            return [1, -1]

        self.__instr__.write(f"{Channel.__channel__}:CRST {Cursor},{Position}")
        return self.__baseclass__.GetAllErrors()

    def GetPlacedCursor(self, Channel: SiglentChannel):
        """
        pySDS [Cursor][GetPlacedCursor] : Return the name of the cursor placed on a channel

            Arguments :
                Channel : The channel to which the cursor belong. This is a SiglentChannel class.

            Returns :
                List of cursor linked to this channel
        """
        return (
            self.__instr__.query(f"{Channel.__channel__}:CRST?")
            .strip()
            .split(" ")
            .split(",")
        )

    def SetCursorType(self, Type):
        """
        pySDS [Cursor][SetCursorType] : Configure the cursor type

            Arguments :
                Type : X | -X | Y | -Y

            Returns :
                Configured type
                or "-1" if wrong type passed
        """
        if Type not in ["X", "-X", "Y", "-Y"]:
            return [1, -1]

        self.__instr__.write(f"CRTY {Type}").strip().split(" ")[-1]
        return self.__baseclass__.GetAllErrors()

    def GetCursorValue(self, Channel: SiglentChannel, Mode):
        """
        pySDS [Cursor][GetCursorValue] : Return the values of a cursor

        WARNING : Make sure that a cursor has been placed on this channel, or the device will trigger an error :

            Arguments :
                Channel : The channel where the measure belong
                Mode : HREL | VREL ==> Read the horizontal or vertical measure

            Returns : (List of values)
                Delta
                (1 / Delta) : Only in HREL mode, 0 in VREL mode
                Value1
                Value2

                or
                [-err, -err, -err, -err] in case or error with err = errror code

            Errors code
                -1 : Error occured while running the command
        """

        ret = (
            self.__instr__.query(f"{Channel.__channel__}:CRVA? {Mode}")
            .strip()
            .split(" ")[-1]
            .split(",")
        )

        match ret[0]:
            case "HREL":
                return [float(ret[1]), float(ret[2]), float(ret[3]), float(ret[4])]
            case "VREL":
                return [float(ret[1]), 0.00, float(ret[2]), float(ret[3])]
            case _:  # default case
                return [-1] * 4
        return [-2, -2, -2, -2]  # shouldn't get here, and if we do, good luck !



# ============================================================================================================
# References.py
# lheywang on 21/12/2024
#
# Base file for the reference class
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase
from ..channel import SiglentChannel


class SiglentReference(SiglentBase):
    """
    pySDS [Files][SiglentReference] :   Class herited from SiglentBase.
                                        Store all command related the control of reference channel selection

        Attributes :
            Herited from SiglentBase

        Methods :
            Private (0) :
                None

            Public (8):
            CloseReference :                Close the reference functions package
            EnableDisplayOfReference :      Enable display
            DisableDisplayOfReference :     Disable display
            SetReferenceLocation :          Set location of the reference (A, B, C, D)
            SetReferenceOffset :            Set reference offset
            SaveWaveformAsReference :       Save waveform as reference
            SetReferenceScale :             Set reference scale
            SetReferenceSource :            Set reference source
    """

    def CloseReference(self):
        """
        pySDS [Reference][CloseReference] : Close the reference function on the device

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("REFCL")
        return self.__baseclass__.GetAllErrors()

    def EnableDisplayOfReference(self):
        """
        pySDS [Reference][EnableDisplayOfReference] : Display the used reference on the screen

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("REFDS ON")
        return self.__baseclass__.GetAllErrors()

    def DisableDisplayOfReference(self):
        """
        pySDS [Reference][DisableDisplayOfReference] : Hide the used reference on the screen

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("REFDS OFF")
        return self.__baseclass__.GetAllErrors()

    def SetReferenceLocation(self, Location):
        """
        pySDS [Reference][SetReferenceLocation] : Set the location of the actual reference

            Arguments :
                Location : A | B | C | D

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid location
        """
        if Location not in ["A", "B", "C", "D"]:
            return [1, -1]

        self.__instr__.write(f"REFLA REF{Location}")
        return self.__baseclass__.GetAllErrors()

    def SetReferenceOffset(self, Offset):
        """
        pySDS [Reference][SetReferenceOffset] : Apply an offset on the reference display

            Arguments :
                Offset : The offset in volts

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write(f"REFPO {Offset}V")
        return self.__baseclass__.GetAllErrors()

    def SaveWaveformAsReference(self):
        """
        pySDS [Reference][SaveWaveformAsReference] : Save the current channel source on the current reference location

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("REFSA")
        return self.__baseclass__.GetAllErrors()

    def SetReferenceScale(self, Scale):
        """
        pySDS [Reference][SetReferenceScale] : Set the display reference scale

            Arguments :
                Scale : The unit in volts (500uV - 10V)

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid scale
        """
        if Scale < 0.000_5 or Scale > 10:
            return [1, -1]

        self.__instr__.write(f"REFSC {Scale}V")
        return self.__baseclass__.GetAllErrors()

    def SetReferenceSource(self, Channel: SiglentChannel):
        """
        pySDS [Reference][SetReferenceSource] : Set the reference channel to be used

            Arguments :
                Channel : SiglentChannel to be used (or MATH)

            Returns :
                self.GetAllErrors()
        """

        if Channel == "MATH":
            self.__instr__.write("REFSR MATH")
        else:
            self.__instr__.write(f"REFSR {Channel.__channel__}")
        return self.__baseclass__.GetAllErrors()



# ============================================================================================================
# Measure.py
# lheywang on 21/12/2024
#
# Base file for the measure class
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase
from ..channel import SiglentChannel


class SiglentMeasure(SiglentBase):
    """
    pySDS [Files][SiglentMeasure] : Class herited from SiglentBase.
                                    Store all command related the control of automated measures

        Attributes :
            Herited from SiglentBase

        Methods :
            Private (0) :
                None

            Public (14):
                GetSignalFrequency :            Get triggering signal frequency
                SetDelayMeasure :               Configure delay measure
                GetDelayMeasure :               Read delay measure
                SetMeasure :                    Configure measure
                GetMeasure :                    Read measure
                EnableMeasureStatistics :       Enable statistics
                DisableMeasureStatistics :      Disable statistics
                ResetMeasureStatistics :        Reset statistics
                RemoveMeasures :                Remove all measures
                GetStatsMeasure :               Read measures (with statistics)
                EnableMeasureGating :           Enable measure gating (restraint the measure to a part of the waveform)
                DisableMeasureGating :          Disable measure gating
                SetGatingLowerLimit :           Configure lower limit for gating
                SetGatingHigherLimit :          Configure higher limit for gating
    """

    def GetSignalFrequency(self):
        """
        pySDS [Measure][GetSignalFrequency] : Get the number of trigger crossing per seconds, in Hz.

        WARNING : Measure only the trigger channel.

            Arguments :
                None

            Returns :
                Measure, in Hz.
        """
        ret = self.__instr__.query("CYMT?").strip().split(" ")[-1]

        if "E" in ret:
            # Notation 1 (seems newer ?)
            return float(ret[:-2])
        else:
            if "<" in ret:
                # Some old notation are <10Hz, with is casted to 10 and returned like that.
                # They removed it in the newest devices, with then use the parsing.
                return 10.00

            val = ret.replace("k", "").replace("M", "")
            val = float(val[:-2].strip())

            if "k" in ret:
                return val * 1_000
            if "M" in ret:
                return val * 1_000_000
            return val

    def SetDelayMeasure(
        self, Type: str, Channel1: SiglentChannel, Channel2: SiglentChannel
    ):
        """
        pySDS [Measure][SetDelayMeasure] : Configure a time related measurement

        WARNING : This function shall be called before calling the Get function.

            Arguments :
                Type :      Type of mesure. PHA | FRR | FRF | FFR | FFF | LRR | LRF | LFR | LFF | SKEW
                Channel1 :  Siglent channel for the source 1
                Channel2 :  Siglent channel for the source 2

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid type
                -2 : Invalid Channel1 class
                -3 : Invalid Channel2 class

            Available measures :
                PHA :The phase difference between two channels. (rising edge - rising edge)
                FRR Delay between two channels. (first rising edge - first rising edge)
                FRF Delay between two channels. (first rising edge - first falling edge)
                FFR Delay between two channels. (first falling edge - first rising edge)
                FFF Delay between two channels. (first falling edge - first falling edge)
                LRR Delay between two channels. (first rising edge - last rising edge)
                LRF Delay between two channels. (first rising edge - last falling edge)
                LFR Delay between two channels. (first falling edge - last rising edge)
                LFF Delay between two channels. (first falling edge - last falling edge)
                SKEW Delay between two channels. (edge to edge of the same type)
        """
        if Type not in [
            "PHA",
            "FRR",
            "FRF",
            "FFR",
            "FFF",
            "LRR",
            "LRF",
            "LFR",
            "LFF",
            "SKEW",
        ]:
            return [1, -1]
        if type(Channel1) is not SiglentChannel:
            return [1, -2]
        if type(Channel2) is not SiglentChannel:
            return [1, -3]

        self.__instr__.write(
            f"MEAD {Type},{Channel1.__channel__}-{Channel2.__channel__}:"
        )
        return self.__baseclass__.GetAllErrors()

    def GetDelayMeasure(
        self, Type: str, Channel1: SiglentChannel, Channel2: SiglentChannel
    ):
        """
        pySDS [Measure][GetDelayMeasure] : Read the value of a measure (and maybe set it ??)

            Arguments :
                Type :      Type of mesure. (See list linked)
                Channel1 :  SiglentChannel for the source 1
                Channel2 :  SiglentChannel for the source 2

            Returns :
                The value read, or if negative :
                -1 : Invalid type
                -2 : Invalid Channel1 class
                -3 : Invalid Channel2 class

            Available measures :
                PHA :The phase difference between two channels. (rising edge - rising edge)
                FRR Delay between two channels. (first rising edge - first rising edge)
                FRF Delay between two channels. (first rising edge - first falling edge)
                FFR Delay between two channels. (first falling edge - first rising edge)
                FFF Delay between two channels. (first falling edge - first falling edge)
                LRR Delay between two channels. (first rising edge - last rising edge)
                LRF Delay between two channels. (first rising edge - last falling edge)
                LFR Delay between two channels. (first falling edge - last rising edge)
                LFF Delay between two channels. (first falling edge - last falling edge)
                SKEW Delay between two channels. (edge to edge of the same type)
        """
        if Type not in [
            "PHA",
            "FRR",
            "FRF",
            "FFR",
            "FFF",
            "LRR",
            "LRF",
            "LFR",
            "LFF",
            "SKEW",
        ]:
            return -1
        if type(Channel1) is not SiglentChannel:
            return -2
        if type(Channel2) is not SiglentChannel:
            return -3

        ret = (
            self.__instr__.query(
                f"{Channel1.__channel__}-{Channel2.__channel__}:MEAD? {Type}"
            )
            .strip()
            .split(",")[-1]
        )
        if "S" in ret:
            # Handle timings
            return float(ret.replace("S", ""))
        else:
            # Handle degrees
            return float(ret.replace("degree", ""))

    def SetMeasure(self, Type: str, Channel: SiglentChannel):
        """
        pySDS [Measure][SetMeasure] : Install a new measure

            Arguments :
                Type :      The measure to be done (see list at the end of the doc)
                Channel :   SiglentChannel to be measured

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid type
                -2 : Invalid Channel type

            Available measures :
                PKPK : vertical peak-to-peak
                MAX : maximum vertical value
                MIN : minimum vertical value
                AMPL : vertical amplitude
                TOP : waveform top value
                BASE : waveform base value
                CMEAN : average value in the first cycle
                MEAN : average value
                STDEV : standard deviation of the data
                VSTD : standard deviation of the data in the first cycle
                RMS : RMS value
                CRMS : RMS value in the first cycle
                OVSN : overshoot of a falling edge
                FPRE : preshoot of a falling edge
                OVSP : overshoot of a rising edge
                RPRE : preshoot of a rising edge
                LEVELX : Level measured at trigger position
                PER : period
                FREQ : frequency
                PWID : positive pulse width
                NWID : negative pulse width
                RISE : rise-time
                FALL : fall-time
                WID : Burst width
                DUTY : positive duty cycle
                NDUTY : negative duty cycle
                DELAY : time from the trigger to the first transition at the 50% crossing
                TIMEL : time from the trigger to each rising edge at the 50% crossing
                ALL : All measurements snapshot, equal to turn on the switch of all measure
        """
        if Type not in [
            "PKPK",
            "MAX",
            "MIN",
            "AMPL",
            "TOP",
            "BASE",
            "CMEAN",
            "MEAN",
            "STDEV",
            "VSTD",
            "RMS",
            "CRMS",
            "OVSN",
            "FPRE",
            "OVSP",
            "RPRE",
            "LEVELX",
            "PER",
            "FREQ",
            "PWID",
            "NWID",
            "RISE",
            "FALL",
            "WID",
            "DUTY",
            "NDUTY",
            "DELAY",
            "TIMEL",
            "ALL",
        ]:
            return [1, -1]
        if type(Channel) is not SiglentChannel:
            return [1, -2]

        self.__instr__.write(f"PACU {Type},{Channel.__channel__}")
        return self.__baseclass__.GetAllErrors()

    def GetMeasure(self, Type: str, Channel: SiglentChannel):
        """
        pySDS [Measure][GetMeasure] : Read a measure

            Arguments :
                Type :      The measure to be done (see list at the end of the doc)
                Channel :   SiglentChannel to be measured

            Returns :
                dict with measure name and value.
                Errors are reaturned under the error key.

            Available measures :
                PKPK : vertical peak-to-peak
                MAX : maximum vertical value
                MIN : minimum vertical value
                AMPL : vertical amplitude
                TOP : waveform top value
                BASE : waveform base value
                CMEAN : average value in the first cycle
                MEAN : average value
                STDEV : standard deviation of the data
                VSTD : standard deviation of the data in the first cycle
                RMS : RMS value
                CRMS : RMS value in the first cycle
                OVSN : overshoot of a falling edge
                FPRE : preshoot of a falling edge
                OVSP : overshoot of a rising edge
                RPRE : preshoot of a rising edge
                LEVELX : Level measured at trigger position
                PER : period
                FREQ : frequency
                PWID : positive pulse width
                NWID : negative pulse width
                RISE : rise-time
                FALL : fall-time
                WID : Burst width
                DUTY : positive duty cycle
                NDUTY : negative duty cycle
                DELAY : time from the trigger to the first transition at the 50% crossing
                TIMEL : time from the trigger to each rising edge at the 50% crossing
                ALL : All measurements snapshot, equal to turn on the switch of all measure
        """
        if Type not in [
            "PKPK",
            "MAX",
            "MIN",
            "AMPL",
            "TOP",
            "BASE",
            "CMEAN",
            "MEAN",
            "STDEV",
            "VSTD",
            "RMS",
            "CRMS",
            "OVSN",
            "FPRE",
            "OVSP",
            "RPRE",
            "LEVELX",
            "PER",
            "FREQ",
            "PWID",
            "NWID",
            "RISE",
            "FALL",
            "WID",
            "DUTY",
            "NDUTY",
            "DELAY",
            "TIMEL",
            "ALL",
        ]:
            return {"error": -1}
        if type(Channel) is not SiglentChannel:
            return {"error": -2}

        out = dict()
        ret = (
            self.__instr__.query(f"{Channel.__channel__}:PAVA? {Type}")
            .strip()
            .split(" ")[-1]
            .split(",")
        )
        for index, meas in enumerate(ret):
            # We need to handle naming and value for each
            if index % 2 == 0:
                key = meas
                val = ret[index + 1]

                # remove traling units
                while val[-1].isalpha():
                    val = val[:-1]

                if "*" in val:
                    val = 0.00

                val = float(val)
                out[key] = val
            else:
                # Continue the loop. We're not right
                continue
        return out

    def EnableMeasureStatistics(self):
        """
        pySDS [Measure][EnableMeasureStatistics] : Enable measurement stats

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("PASTAT ON")
        return self.__baseclass__.GetAllErrors()

    def DisableMeasureStatistics(self):
        """
        pySDS [Measure][DisableMeasureStatistics] : Disable measurement stats

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("PASTAT OFF")
        return self.__baseclass__.GetAllErrors()

    def ResetMeasureStatistics(self):
        """
        pySDS [Measure][ResetMeasureStatistics] : Reset measurement stats values

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("PASTAT RESET")
        return self.__baseclass__.GetAllErrors()

    def RemoveMeasures(self):
        """
        pySDS [Measure][RemoveMeasures] : Remove installed measures

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("PASTAT")
        return self.__baseclass__.GetAllErrors()

    def GetStatsMeasure(self, MeasureID):
        """
        pySDS [Measure][GetStatsMeasure] : Read the statistics for a measure.

        WARNING : This function need statistics to be enabled !
        WARNING2 : The ID of the measure

            Arguments :
                NumberOfMeasures : The number of measured stats wanted.

            Returns :
                dict with measure name and value.
                Errors are reaturned under the error key.

            Errors codes :
                -1 : Invalid Number of measures. Must be between 1 and 5

        """
        if MeasureID < 0 or MeasureID > 5:
            return {"errors": -1}

        out = dict()
        ret = (
            self.__instr__.query(f"PAVA? STAT{MeasureID}").strip().split(" ")[2:]
        )  # Two last elements
        out["channel"] = ret[0]

        measures = ret[1].split(":")
        out["type"] = measures[0]

        values = measures[1].split(",")

        for index, meas in enumerate(values):
            # We need to handle naming and value for each
            if index % 2 == 0:
                key = meas
                val = values[index + 1]

                # remove traling units
                while val[-1].isalpha():
                    val = val[:-1]

                if "*" in val:
                    val = 0.00

                val = float(val)
                out[key] = val
            else:
                # Continue the loop. We're not on the correct index
                continue
        return out

    def EnableMeasureGating(self):
        """
        pySDS [Measure][EnableMeasureGating] : Enable measurement gating, to only consider a small part of the waveform

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("MEGS ON")
        return self.__baseclass__.GetAllErrors()

    def DisableMeasureGating(self):
        """
        pySDS [Measure][DisableMeasureGating] : Disable measurement gating.

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("MEGS OFF")
        return self.__baseclass__.GetAllErrors()

    def SetGatingLowerLimit(self, Time: float):
        """
        pySDS [Measure][SetGatingLowerLimit] : Set lower gating time.

        WARNING : This value can't be greater than Higher limit. No checks are done !

            Arguments :
                Time : Value of time to be used, in seconds.

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write(f"MEGA {Time}")
        return self.__baseclass__.GetAllErrors()

    def SetGatingHigherLimit(self, Time: float):
        """
        pySDS [Measure][SetGatingLowerLimit] : Set higher gating time.

        WARNING : This value can't be lower than Lower limit. No checks are done !

            Arguments :
                Time : Value of time to be used, in seconds.

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write(f"MEGB {Time}")
        return self.__baseclass__.GetAllErrors()



# ============================================================================================================
# UART.py
# lheywang on 02/01/2025
#
# Advanced file for the trigger class, specialized on Serial bus
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase
from ..channel import SiglentChannel
from ..digital import SiglentDChannel


class SiglentUART(SiglentBase):
    """
    pySDS [Trigger][SiglentUART] :  Class herited from SiglentBase.
                                    Store all command related the control of the triggering system for the UART bus

        Attributes :
            Herited from SiglentBase

        Methods :
            Private (0) :
                None

            Public (12):
                SetTriggerOnRX :                Set trigger on RX Pin
                SetTriggerOnTX :                Set trigger on TX Pin
                SetTriggerDataSource :          Set used data source
                SetTriggerCondition :           Set trigger condition
                SetTriggerQualifier :           Set trigger qualifier
                ConfigureTriggerData :          Set triggering data
                SetTriggerBaud :                Set triggering baud (act also on bus decoding)
                ConfigureTriggerDataLen :       Set trigger data len (act also on bus decoding)
                ConfigureTriggerParity :        Set trigger parity (act also on bus decoding)
                ConfigureTriggerPolarity :      Set trigger polarity (act also on bus decoding)
                ConfigureTriggerStop :          Set trigger stop bits (act also on bus decoding)
                ConfigureTriggerBitOrder :      Set trigger bit order (act also on bus decoding)
    """

    def SetTriggerOnRX(self, Channel, Threshold=1.65):
        """
        pySDS [UART][SetTriggerOnRX] : Configure the trigger on the RX pin

            Arguments :
                Channel :       SiglentChannel or SiglentDCHannel related to the RX pin
                Threshold :     For analog channel only, the used voltage. Default to 1.65

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Channel
        """
        if type(Channel) is not SiglentChannel and type(Channel) is not SiglentDChannel:
            return [1, -1]

        cmd = f"TRUART:RX {Channel.__channel__}"

        if type(Channel) == SiglentChannel:
            cmd += f",{Threshold}"

        self.__instr__.write(cmd)
        return self.__baseclass__.GetAllErrors()

    def SetTriggerOnTX(self, Channel, Threshold=1.65):
        """
        pySDS [UART][SetTriggerOnTX] : Configure the trigger on the TX pin

            Arguments :
                Channel :       SiglentChannel or SiglentDCHannel related to the TX pin
                Threshold :     For analog channel only, the used voltage. Default to 1.65

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Channel
        """
        if type(Channel) is not SiglentChannel and type(Channel) is not SiglentDChannel:
            return [1, -1]

        cmd = f"TRUART:TX {Channel.__channel__}"

        if type(Channel) == SiglentChannel:
            cmd += f",{Threshold}"

        self.__instr__.write(cmd)
        return self.__baseclass__.GetAllErrors()

    def SetTriggerDataSource(self, Source: str):
        """
        pySDS [UART][SetTriggerDataSource] : Configure the used data source to trigger

            Arguments :
                Source : Data Source used RX | TX

            Returns :
                self.GetAllErrors
                or
                -1 : Invalid Source
        """
        if Source not in ["RX", "TX"]:
            return [1, -1]

        self.__instr__.write(f"TRUART:TRTY {Source}")
        return self.__baseclass__.GetAllErrors()

    def SetTriggerCondition(self, Condition):
        """
        pySDS [UART][SetTriggerCondition] : Set the trigger on a specific case

            Arguments :
                Condition : START | STOP | DATA | ERROR

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Condition
        """
        if Condition not in ["START", "STOP", "DATA", "ERROR"]:
            return [1, -1]

        self.__instr__.write(f"TRUART:CON {Condition}")
        return self.__baseclass__.GetAllErrors()

    def SetTriggerQualifier(self, Qualifier):
        """
        pYSDS [UART][SetTriggerQualifier] : Set the UART Qualifier condition

            Arguments :
                Qualifier : EQUAL | MORE | LESS

            Returns
                self.GetAllErrors()
                or
                -1 : Invalid qualifier
        """
        if Qualifier not in ["MORE", "LESS", "EQUAL"]:
            return [1, -1]

        self.__instr__.write(f"TRUART:QUAL {Qualifier}")

    def ConfigureTriggerData(self, Data):
        """
        pySDS [UART][ConfigureTriggerData] : Configure triggering data

            Arguments :
                Data : Data to be used

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid value
        """
        if int(Data) < 0 or int(Data) > 256:
            return [1, -1]

        self.__instr__.write(f"TRUART:DATA {Data}")
        return self.__baseclass__.GetAllErrors()

    def SetTriggerBaud(self, Baud):
        """
        pySDS [UART][SetTriggerBaud] : Set the baud rate of the UART comm

            Arguments :
                Baud : Value.

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Baud
        """
        if Baud < 300 or Baud > 5_000_000:
            return [1, -1]

        if Baud not in [600, 1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200]:
            cmd = f"TRUART:BAUD CUSTOM,{Baud}"
        else:
            cmd = f"TRUART:BAUD {Baud}"

        self.__instr__.write(cmd)
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerDataLen(self, Len: int):
        """
        pySDS [UART][ConfigureTriggerDataLen] : Configure the number of bits to trigger

            Arguments :
                Len : 5 to 8

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Len
        """
        if Len < 5 or Len > 8:
            return [1, -1]

        self.__instr__.write(f"TRUART:DLEN {Len}")
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerParity(self, Parity: str):
        """
        pySDS [UART][ConfigureTriggerParity] : Configure the parity bit used

            Arguments :
                Parity : ODD | EVEN | NONE

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Parity
        """
        if Parity not in ["ODD", "EVEN", "NONE"]:
            return [1, -1]

        self.__instr__.write(f"TRUART:PAR {Parity}")
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerPolarity(self, Polarity: str):
        """
        pySDS [UART][ConfigureTriggerPolarity] : Configure the bus polarity

            Arguments :
                Polarity : LOW | HIGH

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Polarity
        """
        if Polarity not in ["LOW", "HIGH"]:
            return [1, -1]

        self.__instr__.write(f"TRUART:POL {Polarity}")
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerStop(self, Stop: float):
        """
        pySDS [UART][ConfigureTriggerStop] : Configure the number of stops bits used

            Arguments :
                Stop : 1 | 1.5 | 2

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid stop bit config
        """
        if Stop not in [1, 1.5, 2]:
            return [1, -1]

        self.__instr__.write(f"TRUART:STOP {Stop}")
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerBitOrder(self, Order: str):
        """
        pySDS [UART][ConfigureTriggerBitOrder] : Configure the bit order on the comm

            Arguments :
                Order : LSB | MSB

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Order
        """
        if Order not in ["LSB", "MSB"]:
            return [1, -1]

        self.__instr__.write(f"TRUART:BIT {Order}")
        return self.__baseclass__.GetAllErrors()



# ============================================================================================================
# Screen.py
# lheywang on 17/12/2024
#
# Base file for the screen management class
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase


class SiglentScreen(SiglentBase):
    """
    pySDS [Display][SiglentScreen] :    Class herited from SiglentBase.
                                        Store all command related the control of display.

        Attributes :
            Herited from SiglentBase

        Methods :
            Private (0) :
                None

            Public (7):
                EnableScreenInterpolation :         Enable continuous display of the tracks
                DisableScreenInterpolation :        Enable the display of only points
                SelectGrid :                        Select the grid type
                SetIntensity :                      Select the trace and grid intensity
                ShowMenu :                          Show menu
                HideMenu :                          Hide menu
                ConfigurePersistence :              Configure trace persistence
    """

    def EnableScreenInterpolation(self):
        """
        pySDS [Screen][EnableScreenInterpolation] : Enable the drawing of lines between data points

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("DTJN ON")
        return self.__baseclass__.GetAllErrors()

    def DisableScreenInterpolation(self):
        """
        pySDS [Screen][DisableScreenInterpolation] : Disable the drawing of lines between data points

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("DTJN OFF")
        return self.__baseclass__.GetAllErrors()

    def SelectGrid(self, Grid):
        """
        pySDS [Screen][SelectGrid] : Select the grid on the display

            Arguments :
                Grid : FULL | HALF | OFF

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid grid mode
        """
        if Grid not in ["FULL", "HALF", "OFF"]:
            return [1, -1]

        self.__instr__.write(f"GRDS {Grid}")
        return self.__baseclass__.GetAllErrors()

    def SetIntensity(self, Grid, Trace):
        """
        pySDS [Screen][Intensity] : Set intensity of the grid display

            Arguments :
                Grid : Value for the grid. 0 to 100
                Trace : Value for the trace. 0 to 100

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid grid value
                -2 : Invalid trace value
        """
        if Grid < 0 or Grid > 100:
            return [1, -1]
        if Trace < 0 or Trace > 100:
            return [1, -2]

        self.__instr__.write(f"INTS GRID,{Grid},TRACE,{Trace}")
        return self.__baseclass__.GetAllErrors()

    def ShowMenu(self):
        """
        pySDS [Screen][ShowMenu] : Show the menu on the screen

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("MENU ON")
        return self.__baseclass__.GetAllErrors()

    def HideMenu(self):
        """
        pySDS [Screen][HideMenu] : Hide the menu on the screen

            Arguments :
                None

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write("MENU OFF")
        return self.__baseclass__.GetAllErrors()

    def ConfigurePersistence(self, Value):
        """
        pySDS [Screen][ConfigurePersistence] : Configure the persistence of the track on the screen

        WARNING : OFF may not be available for all of the models.

            Arguments :
                Value : INFINITE | 1 | 5 | 10 | 30 | (OFF)

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid persistence value
        """
        if Value not in ["INFINITE", "1", "5", "10", "30", "OFF"]:
            return [1, -1]

        self.__instr__.write(f"PESU {Value}")
        return self.__baseclass__.GetAllErrors()



# ============================================================================================================
# Digital.py
# lheywang on 21/12/2024
#
# Base file for the digital class
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase


class SiglentDChannel(SiglentBase):
    def __init__(self, instr, baseclass, channel):
        """
        Overhide the standard class init to store some more advanced data !

        Check SiglentBase doc before !

        Added attributes :
            Private (1) :
                __channel__ :   Descriptor of the channel

            Public (0) :
                None

        Added methods :
            Private (0) :
                None

            Public (0) :
                None
        """
        super().__init__(instr, baseclass)
        self.__channel__ = channel



# ============================================================================================================
# Trigger.py
# lheywang on 19/12/2024
#
# Base file for the trigger class
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase

from .IIC import SiglentIIC
from .SPI import SiglentSPI
from .CAN import SiglentCAN
from .UART import SiglentUART
from .LIN import SiglentLIN


class SiglentTrigger(SiglentBase):
    """
    pySDS [Files][SiglentTrigger] : Class herited from SiglentBase.
                                    Store all command related the control of the triggering system

                                    Due to advanced features available, this class group subclasses.
                                    Thus, it's possible to trigger on serial busses for a specific address or conditions.

    WARNING : Advanced features are linked to bus decoding ability, and can sometimes interfer between their configurations !

        Attributes :
            Herited from SiglentBase
            +
            I2C (SiglentIIC Class), specified for I2C operation
            SPI (SiglentSPI Class), specified for SPI operation
            LIN (SiglentLIN Class), specified for LIN operation
            SERIAL (SiglentUART Class), specified for UART operation
            CAN (SiglentCAN Class), specified for CAN Operation

        Methods :
            Private (0) :
                None

            Public (15):
                SetCoupling :   Configure trigger coupling
                SetDelay :      Configure trigger delay
                GetDelay :      Get trigger delay
                SetLevel1 :     Set threshold 1
                SetLevel2 :     Set threshold 2
                SetMode :       Set trigger mode
                GetMode :       Get trigger mode
                SetSelect :     Set select
                GetSelect :     Get trigger select
                SetSlope :      Set trigger slope
                GetSlope :      Get trigger slope
                SetWindow :     Set trigger Window
                GetWindow :     Get trigger Window
                SetPattern :    Set trigger pattern
                GetPattern :    Get trigger pattern
    """

    def __init__(self, instr, baseclass, number=0):
        """
        Overhide the standard class init to store some more advanced data !

        Check SiglentBase doc before !

        Added attributes :
            Private (0) :

            Public (0) :
                I2C (SiglentIIC Class), specified for I2C operation
                SPI (SiglentSPI Class), specified for SPI operation
                LIN (SiglentLIN Class), specified for LIN operation
                SERIAL (SiglentUART Class), specified for UART operation
                CAN (SiglentCAN Class), specified for CAN Operation

        Added methods :
            Private (0) :
                None

            Public (0) :
                None
        """
        # Calling default init
        super().__init__(instr, baseclass, number)

        # Creating new members, which are also herited from the base class
        self.I2C = SiglentIIC(instr, baseclass, number)
        self.SPI = SiglentSPI(instr, baseclass, number)
        self.LIN = SiglentLIN(instr, baseclass, number)
        self.CAN = SiglentCAN(instr, baseclass, number)
        self.Serial = SiglentUART(instr, baseclass, number)

        return

    #
    #   COUPLING
    #

    def SetCoupling(self, Channel, Mode):
        """
        PySDS [Trigger][SetCoupling] :  Configure the source and coupling of the trigger

        WARNING : The command to know the state of the trigger hasn't been developped since it suppose we know the channel used...

            Arguments :
                Channel : C1 | C2 | C3 | C4 | EX | EX5 | LINE : You can pass a member of the ENUM TriggerSources, or it's string. (Warning : Make sure to remain consistent, otherwise the last channel used will be used)
                Mode : AC | DC | HFREF | LFREJ : You can pass a member of the ENUM TriggerModes or it's name direcly

            Returns :
                self.GetAllErrors() : List of errors
        """
        if Channel not in ["C1", "C2", "C3", "C4", "EX", "EX5", "LINE"]:
            return [1, -1]  # Emulate the standard return type

        if Mode not in ["AC", "DC", "HFREJ", "LFREJ"]:
            return [1, -2]  # Emulate the standard return type

        self.__instr__.write(f"{Channel}:TRCP {Mode}")
        return self.__baseclass__.GetAllErrors()

    #
    #   DELAY
    #

    def SetDelay(self, Delay: float):
        """
        PySDS [Trigger][SetDelay] :  Configure the delay (may be positive or negatives)* between the trigger and the first acquistion

        WARNING : Positive delay are only supported on some devices.

            Arguments :
                Delay : The delay in ms to apply

            Returns :
                self.GetAllErrors() : List of errors
        """
        self.__instr__.write(f"TRDL {Delay}ms")
        return self.__baseclass__.GetAllErrors()

    def GetDelay(self):
        """
        PySDS [Trigger][GetDelay] :  Read the delay applied between trigger and acquisition

            Arguments :
                None

            Returns :
                Float : The number of ms of delay
        """
        return float(self.__instr__.query("TRDL?").strip().split(" ")[-1][:-1]) * 1000

    #
    #   LEVEL
    #

    def SetLevel1(self, Channel, Value: float):
        """
        PySDS [Trigger][SetLevel1] :  Set the level of the specified trigger for a specific channel

            Arguments :
                Channel : C1 | C2 | C3 | C4 | EX | EX5 | LINE : You can pass a member of the ENUM TriggerSources, or it's string. (Warning : Make sure to remain consistent, otherwise the last channel used will be used)
                Value : The value in V where to place the trigger

            Returns :
                self.GetAllErrors() : List of errors
        """
        self.__instr__.write(f"{Channel}:TRLV {Value}")
        return self.__baseclass__.GetAllErrors()

    def SetLevel2(self, Channel, Value: float):
        """
        PySDS [Trigger][SetLevel2] :  Set the level of the specified trigger for a specific channel

        WARNING : This function is not available on SPO devices

            Arguments :
                Channel : C1 | C2 | C3 | C4 | EX | EX5 | LINE : You can pass a member of the ENUM TriggerSources, or it's string. (Warning : Make sure to remain consistent, otherwise the last channel used will be used)
                Value : The value in V where to place the trigger

            Returns :
                self.GetAllErrors() : List of errors
        """
        self.__instr__.write(f"{Channel}:TRLV2 {Value}")
        return self.__baseclass__.GetAllErrors()

    #
    #   MODE
    #

    def SetMode(self, Mode):
        """
        PySDS [Trigger][SetMode] :  Configure the mode of operation of the trigger

            Arguments :
                Mode : AUTO | NORM | SINGLE | STOP : Restrained to theses values by an enum.

            Returns :
                Float : The number of ms of delay
        """
        if Mode not in ["AUTO", "NORM", "SINGLE", "STOP", "STOP"]:
            return [1, -1]  # Emulate the standard return type

        self.__instr__.write(f"TRMD {Mode}")
        return self.__baseclass__.GetAllErrors()

    def GetMode(self):
        """
        PySDS [Trigger][GetMode] :  Read the mode of operation of the trigger

            Arguments :
                None

            Returns :
                String : The mode
        """
        return self.__instr__.query("TRMD?").strip().split(" ")[-1]

    #
    #   SELECT
    #

    def SetSelect(self, *args):
        """
        PySDS [Trigger][SetSelect] :  Configure the trigger for very advanced usages.

        WARNING :   Due to the very advanced usage of this function, and the poor traduction / updates of the documentation, I'm currently unable to provide checking.
                    Thus, the function will only pass settings as given, without even trying to make a compatibility check.

            Arguments :
                None

            Returns :
                self.GetAllErrors() : List of errors
        """
        cmd = ""
        for arg in args:
            cmd += arg + ","

        self.__instr__.write(f"TRSE {cmd}")
        return self.__baseclass__.GetAllErrors()

    def GetSelect(self):
        """
        PySDS [Trigger][GetSelect] :    Read the trigger select configuration

        WARNING : Due to the complexity of this function, and the lack of proper traduction / explanations, this function only return a string.

            Arguments :
                None

            Returns :
                String :Command output
        """
        return self.__instr__.query("TRSE?").strip().split(" ")[-1]

    #
    #   SLOPE
    #

    def SetSlope(self, Channel, Slope):
        """
        PySDS [Trigger][SetSlope] :  Configure the 'orientation' of the edge used to trigger.

            Arguments :
                Channel : The channel used for trigger. (Warning : Make sure to remain consistent, otherwise the last channel used will be used)
                Slope : NEG | POS | WINDOW : The edge used to trigger

            Returns :
                self.GetAllErrors() : List of errors TRSL
        """
        if Channel not in ["C1", "C2", "C3", "C4", "EX", "EX5", "LINE"]:
            return [1, -1]  # Emulate the standard return type
        if Slope not in ["POS", "NEG", "WINDOWS"]:
            return [1, -2]  # Emulate the standard return type

        self.__instr__.write(f"{Channel}:TRSL {Slope}")
        return self.__baseclass__.GetAllErrors()

    def GetSlope(self, Channel):
        """
        PySDS [Trigger][GetSlope] :  Return the configured slope for the trigger

            Arguments :
                Channel : The channel used for trigger. (Warning : Make sure to remain consistent, otherwise the last channel used will be used)

            Returns :
                String : The slope used
        """
        if Channel not in ["C1", "C2", "C3", "C4", "EX", "EX5", "LINE"]:
            return [1, -1]  # Emulate the standard return type

        return self.__instr__.query(f"{Channel}:TRSL?").strip().split(" ")[-1][:3]

    #
    #   WINDOW
    #

    def SetWindow(self, Value: float):
        """
        PySDS [Trigger][SetWindow] :  Set the height of the Window used for trigger

            Arguments :
                Value (float) : The value in volt

            Returns :
                self.GetAllErrors() : List of errors
        """
        self.__instr__.write(f"TRWI {Value}V")
        return self.__baseclass__.GetAllErrors()

    def GetWindow(self):
        """
        PySDS [Trigger][GetWindow] :  Get the height of the trigger window

            Arguments :
                None

            Returns :
                Value in volt (float)
        """
        return float(self.__instr__.query("TRWI?").strip().split(" ")[-1][:-1])

    #
    #   PATTERN
    #

    def SetPattern(self, Sources: list, Status: list, Pattern):
        """
        PySDS [Trigger][SetPattern] :  Configure a triggering pattern (Enable multi channel triggering)

            Arguments :
                Source : List of the sources used for the operation. Can only be C1 | C2 | C3 | C4
                Status : List of the status for each source : X | L | H (X = don't care)
                Pattern : AND | OR | NAND | NOR

            Returns :
                self.GetAllErrors() : List of errors
        """
        for Source in Sources:
            if Source not in ["C1", "C2", "C3", "C4"]:
                return [1, -1]  # Emulate the standard return type

        for State in Status:
            if State not in ["X", "L", "H"]:
                return [1, -2]  # Emulate the standard return type

        if Pattern not in ["AND", "OR", "NAND", "NOR"]:
            return [1, -3]  # Emulate the standard return type

        if len(Sources) != len(Status):
            return [1, -3]  # Emulate the standard return type

        if len(Sources) < 1:
            return [1, -3]  # Emulate the standard return type

        cmd = ""
        for index in range(len(Sources)):
            cmd += Sources[index] + "," + Status[index] + ","

        self.__instr__.write(f"TRPA {cmd}STATE,{Pattern}")
        return self.__baseclass__.GetAllErrors()

    def GetPattern(self):
        """
        PySDS [Trigger][GetPattern] : Read the used pattern trigger

            Arguments :
                None

            Returns :
                List of Channel, Conditions and Pattern
        """

        Ret = self.__instr__.query("TRPA?").strip().split(" ")[-1].split(",")

        Pattern = Ret[-1]
        Sources = []
        Conditions = []

        for index in range(0, 2 * int((len(Ret) - 2) / 2), 2):
            if Ret[index + 1] != "X":
                Sources.append(Ret[index])
                Conditions.append(Ret[index + 1])

        return Sources, Conditions, Pattern



# ============================================================================================================
# Communication.py
# lheywang on 21/12/2024
#
# Base file for the Communication class
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase


class SiglentCommunication(SiglentBase):
    """
    pySDS [Communication][SiglentCommunication] :   Class herited from SiglentBase.
                                                    Store all command related to the communication bus
        Attributes :
            Herited from SiglentBase

        Methods :
            Private (0) :
                None

            Public (2):
                SetCommHeader :      Configure the form of response of the device
                GetCommHeader :      Return the form of response of the device
    """

    def SetCommHeader(self, Mode: str):
        """
        SDSpy [Communication][SetCommHeader] :  Configure the used form to answer for the device.

        WARNING :   This function may cause others function to become broken since the parsing from the default answer.
                    LONG / SHORT won't cause issues, the real issue is with OFF where the unit is suppressed. Since the parsing remove the last char, you will end up with power errors !

            Arguments :
                Mode : LONG | SHORT | OFF : The mode of response

            Returns :
                self.GetAllErrors() : List of errors
        """

        if Mode not in ["LONG", "SHORT", "OFF"]:
            return [1, -1]  # Emulate the standard return type

        self.__instr__.write(f"COMM_HEADER {Mode}")
        return self.__baseclass__.GetAllErrors()

    def GetCommHeader(self):
        """
        SDSpy [Communication][GetCommHeader] :  Return the response form of the device

            Arguments :
                None

            Returns :
                String : The mode of operation
        """

        return self.__instr__.query("COMM_HEADER?").strip().split(" ")[-1]



# ============================================================================================================
# Waveform.py
# lheywang on 21/12/2024
#
# Base file for the waveform class
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase
from ..channel import SiglentChannel
from ..digital import SiglentDChannel


class SiglentWaveform(SiglentBase):
    """
    pySDS [Files][SiglentWaveform] :    Class herited from SiglentBase.
                                        Store all command related the control of waveform storage

        Attributes :
            Herited from SiglentBase

        Methods :
            Private (0) :
                None

            Public (1):
                GetWaveformData :       Read the waveform data
    """

    def GetWaveformData(
        self, Channel, Vdiv=1, Vos=0, Tdiv=0.000_000_001, SampleNumber=1000
    ):
        """
        pySDS [Waveform][GetWaveformData] : Return the raw data for a waveform

        WARNING :   Due to high complexity in the formulaes, this function set the export to it's own settings (max number of points !)
                    Otherwise, this would cause issues on the parsing of timebases and so...

            Arguments :
                Channel :   Channel to be exported (Cx, Dx, MATH)
                Vdiv :      Voltage scale for the selected channel      Default to 1
                Vos :       Voltage offset for the selected channel     Default to 0
                Tdiv :      Timebase value                              Default to 1 ns
                SampleNumber : Number of sample in memory (SANU) :      Default to 1000

            Returns :
                List of points (Voltages)
                or
                -1000 : Invalid channel
                -1001 : Did not fetch the end of message
                -1002 : Fetched invalid header
                -1003 : Fetched wrong channel
                -1004 : Fetched incorrect number of bytes
        """
        # Ensure correct arguments
        if (
            type(Channel) is not SiglentChannel
            and type(Channel) is not SiglentDChannel
            and Channel != "MATH"
        ):
            return [-1000], []

        # Configure the device
        self.__instr__.write("WFSU SP,0,NP,0,FP,0")  # All data points

        # First, get the data
        self.__instr__.write(f"{Channel}:WF? DAT2")
        data = self.__instr__.read_raw()

        # Split data
        if Channel == "MATH":
            header = data[:23]
            wave = data[24:-2]
            end = data[-2:]
        else:
            header = data[:21]
            wave = data[22:-2]
            end = data[-2:]

        # End check
        if end[0] != r"\n" and end[1] != r"\n":
            return [-1001], []

        # Header check
        if Channel == "MATH":
            if header[4:15] != ":WF DAT2,#9":
                return [-1002], []
            if header[:4] is not Channel[:4]:
                return [-1003], []
            data_len = int(header[16:])
        else:
            if header[2:13] != ":WF DAT2,#9":
                return [-1002], []
            if header[:2] is not Channel[:2]:
                return [-1003], []
            data_len = int(header[13:])

        if len(wave) != data_len:
            return [-1004], []

        # Analog channel
        if header[0] == "C":
            # Now starting to decode the data
            out = []
            time = []
            for index, byte in enumerate(data):
                val = int(byte, 16)
                if val > 127:
                    val -= 256

                out.append(float(val * (Vdiv / 25) - Vos))
                time.append(float((-Tdiv * 7) + index * Tdiv))

        # Digital channel
        elif header[0] == "D":
            # Now starting to decode the data
            out = []
            time = []
            for index, byte in enumerate(data):
                # Iterate over each bit
                for index2, bit in enumerate(format(int(byte, 16), "#010b")[2:]):
                    out.append(float(bit))
                    time.append(float((-Tdiv * 7) + (index * 8 + index2) * Tdiv))

        # Maths data
        else:
            # Now starting to decode the data
            Tinterpol = data_len / SampleNumber * Tdiv

            out = []
            time = []
            for index, byte in enumerate(data):
                val = int(byte, 16)
                if val > 127:
                    val -= 256

                out.append(float(val * (Vdiv / 25) - Vos))
                time.append(float((-Tdiv * 7) + index * Tinterpol))



# ============================================================================================================
# IIC.py
# lheywang on 02/01/2025
#
# Advanced file for the trigger class, specialized on IIC bus
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase
from ..channel import SiglentChannel
from ..digital import SiglentDChannel


class SiglentIIC(SiglentBase):
    """
    pySDS [Trigger][SiglentIIC] :   Class herited from SiglentBase.
                                    Store all command related the control of the triggering system for the I2C bus

        Attributes :
            Herited from SiglentBase

        Methods :
            Private (0) :
                None

            Public (10):
                SetTriggerOnSCL :               Set the trigger on the SCL Bus
                SetTriggerOnSDA :               Set the trigger on the SDA Bus
                SetTriggerOnCondition :         Set the trigger on advanced conditions
                ConfigureTriggerAddress :       Set the trigger address condition
                ConfigureTriggerData1 :         Set the trigger data 1 condition
                ConfigureTriggerData2 :         Set the trigger data 2 condition
                ConfigureTriggerQual :          Set the trigger eeprom qualifier condition
                ConfigureTriggerRW :            Set the read / write condition
                ConfigureTriggerAddressLen :    Set the address len condition
                ConfigureTriggerDataLen :       Set the data len condition
    """

    def SetTriggerOnSCL(self, Channel, Threshold=1.65):
        """
        pySDS [IIC][SetTriggerOnSCL] : Configure the trigger on the SCL bus.

            Arguments :
                Channel :       SiglentChannel or SiglentDCHannel related to the SCL pin
                Threshold :     For analog channel only, the used voltage. Default to 1.65

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Channel
        """
        if type(Channel) is not SiglentChannel and type(Channel) is not SiglentDChannel:
            return [1, -1]

        cmd = f"TRIIC:SCL {Channel.__channel__}"

        if type(Channel) == SiglentChannel:
            cmd += f",{Threshold}"

        self.__instr__.write(cmd)
        return self.__baseclass__.GetAllErrors()

    def SetTriggerOnSDA(self, Channel, Threshold=1.65):
        """
        pySDS [IIC][SetTriggerOnSDA] : Configure the trigger on the SDA bus.

            Arguments :
                Channel :       SiglentChannel or SiglentDCHannel related to the SDA pin
                Threshold :     For analog channel only, the used voltage. Default to 1.65

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Channel
        """
        if type(Channel) is not SiglentChannel and type(Channel) is not SiglentDChannel:
            return [1, -1]

        cmd = f"TRIIC:SDA {Channel.__channel__}"

        if type(Channel) == SiglentChannel:
            cmd += f",{Threshold}"

        self.__instr__.write(cmd)
        return self.__baseclass__.GetAllErrors()

    def SetTriggerOnCondition(self, Condition: str):
        """
        pySDS [IIC][SetTriggerOnCondition] : Set the trigger on a bus transfer condition

        WARNING : Once the condition is selected, further configuration may be needed.

            Arguments :
                Condition : The selected condition

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid condition

            Possibles conditions :
                START : Start condition.
                STOP : Stop condition.
                RESTART : Another start condition occurs before a stop condition.
                NOACK : Missing acknowledge.
                EEPROM : EEPROM frame containing (Start:Controlbyte:R:Ack:Data).
                7ADDA : 7-bit address frame containing (Start:Address7:R/W:Ack:Data:Data2).
                10ADDA : 10-bit address frame containing (Start:Address10:R/W:Ack:Data:Data2).
                DALENTH : specifie a search based on address lengthand data length.
        """
        if Condition not in [
            "START",
            "STOP",
            "RESTART",
            "NOACK",
            "EEPROM",
            "7ADDA",
            "10ADDA",
            "DALENTH",
        ]:
            return [1, -1]

        self.__instr__.write(f"TRIIC:CON {Condition}")

    def ConfigureTriggerAddress(self, Address: int):
        """
        pySDS [IIC][ConfigureTriggerAddress] : Set the trigger on a specific address on the bus (7ADDA or 10ADDA only)

        WARNING : A condition must be configured to access to this function

            Arguments :
                Address : The device address, 0-128 or 0-1024 (7 or 10 bit address). Address is casted to integer prior device write.

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write(f"TRIIC:ADDR {int(Address)}")
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerData1(self, Data: int):
        """
        pySDS [IIC][ConfigureTriggerData1] : Set the first byte of data on the bus to be filtered (7ADDA or 10ADDA only)

        WARNING : A condition must be configured to access to this function

            Arguments :
                Data : The data, 0-256. Data is casted to integer prior device write.

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write(f"TRIIC:DATA {Data}")
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerData2(self, Data: str):
        """
        pySDS [IIC][ConfigureTriggerData2] : Set the second byte of data on the bus to be filtered (7ADDA or 10ADDA only)

        WARNING : A condition must be configured to access to this function

            Arguments :
                Data : The data, 0-256. Data is casted to integer prior device write.

            Returns :
                self.GetAllErrors()
        """
        self.__instr__.write(f"TRIIC:DAT2 {Data}")
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerQual(self, Qual: str):
        """
        pySDS [IIC][ConfigureTriggerQual] : Configure the qualifier on the bus (EEPROM mode only)

        WARNING : A condition  must be configured to access to this function

            Arguments :
                Qaulifier : EQUAL | MORE | LESS

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid qualifier
        """

        if Qual not in ["MORE", "LESS", "EQUAL"]:
            return [1, -1]

        self.__instr__.write(f"TRIIC:QUAL {Qual}")
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerRW(self, RW: str):
        """
        pySDS [IIC][ConfigureTriggerRW]: Configure the trigger to detect read or write operations

        WARNING : A condition must be configured to access to this function

            Arguments :
                RW : READ | WRITE | DONT_CARE

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid RW
        """
        if RW not in ["READ", "WRITE", "DONT_CARE"]:
            return [1, -1]

        self.__instr__.write(f"TRIIC:RW {RW}")
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerAddressLen(self, Addr: str):
        """
        pySDS [IIC][ConfigureTriggerAddressLen] : Configure the size of the address to be waited. (Data Len mode only)

        WARNING : A condition must be configured to access to this function

            Arguments :
                Addr : 7BIT | 10BIT

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Address len
        """
        if Addr not in ["7BIT", "10BIT"]:
            return [1, -1]

        self.__instr__.write(f"TRIIC:ALEN {Addr}")
        return self.__baseclass__.GetAllErrors()

    def ConfigureTriggerDataLen(self, Len: int):
        """
        pySDS [IIC][ConfigureTriggerDataLen] : Configure the number of byte in the message to trigger. (Data Len mode only)

        WARNING : A condition must be configured to access to this function

            Arguments :
                Len : Integer, between 1 and 12

            Returns :
                self.GetAllErrors()
                or
                -1 : Invalid Len
        """
        if Len < 1 or Len > 12:
            return [1, -1]

        self.__instr__.write(f"TRIIC:DLEN {Len}")
        return self.__baseclass__.GetAllErrors()



# ============================================================================================================
# Cursor.py
# lheywang on 19/12/2024
#
# Base file for the cursor class
#
# ============================================================================================================
from ..BaseOptionnalClass import SiglentBase


class ACAL(SiglentBase):
    def EnableAutomaticCalibration(self):
        """
        PySDS [EnableAutomaticCalibration] :    Enable automatic calibration of the device. (When ? )

        WARNING : This command is only available on some CFL series devices

            Arguments :
                None

            Returns :
                self.GetAllErrors() : List of errors
        """

        self.__instr__.write("ACAL ON")
        return self.__baseclass__.GetAllErrors()

    def DisableAutomaticCalibration(self):
        """
        PySDS [DisableAutomaticCalibration] :    Disable automatic calibration of the device.

        WARNING : This command is only available on some CFL series devices

            Arguments :
                None

            Returns :
                self.GetAllErrors() : List of errors
        """

        self.__instr__.write("ACAL OFF")
        return self.__baseclass__.GetAllErrors()

    def GetAutomaticCalibrationState(self):
        """
        PySDS [GetAutomaticCalibrationState] :   Return the state of the autocalibration

        WARNING : This command is only available on some CFL series devices

            Arguments :
                None

            Returns :
                True | False if enabled | Disabled
        """

        Ret = self.__instr__.write("ACAL?").strip().split(" ")[-1]
        if Ret == "ON":
            return True
        return False
